@@ -15,11 +15,14 @@
  * You should have received a copy of the GNU General Public License
  * along with MiniDLNA. If not, see <http://www.gnu.org/licenses/>.
  */
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <libgen.h>
@@ -103,7 +106,8 @@ update_if_album_art(const char *path)
 	DIR *dh;
 	struct dirent *dp;
 	enum file_types type = TYPE_UNKNOWN;
-	sqlite_int64 art_id = 0;
+	int64_t art_id = 0;
+	int ret;
 
 	strncpyt(fpath, path, sizeof(fpath));
 	match = basename(fpath);
@@ -149,7 +153,8 @@ update_if_album_art(const char *path)
 			DPRINTF(E_DEBUG, L_METADATA, "New file %s looks like cover art for %s\n", path, dp->d_name);
 			snprintf(file, sizeof(file), "%s/%s", dir, dp->d_name);
 			art_id = find_album_art(file, NULL, 0);
-			if( sql_exec(db, "UPDATE DETAILS set ALBUM_ART = %lld where PATH = '%q'", art_id, file) != SQLITE_OK )
+			ret = sql_exec(db, "UPDATE DETAILS set ALBUM_ART = %lld where PATH = '%q'", (long long)art_id, file);
+			if( ret != SQLITE_OK )
 				DPRINTF(E_WARN, L_METADATA, "Error setting %s as cover art for %s\n", match, dp->d_name);
 		}
 	}
@@ -267,6 +272,7 @@ check_for_album_file(const char *path)
 	char *art_file;
 	const char *dir;
 	struct stat st;
+	int ret;
 
 	if( stat(path, &st) != 0 )
 		return NULL;
@@ -281,20 +287,28 @@ check_for_album_file(const char *path)
 
 	/* First look for file-specific cover art */
 	snprintf(file, sizeof(file), "%s.cover.jpg", path);
-	if( access(file, R_OK) == 0 )
+	ret = access(file, R_OK);
+	if( ret != 0 )
 	{
-		if( art_cache_exists(file, &art_file) )
-			goto existing_file;
-		free(art_file);
-		imsrc = image_new_from_jpeg(file, 1, NULL, 0, 1, ROTATE_NONE);
-		if( imsrc )
-			goto found_file;
+		strncpyt(file, path, sizeof(file));
+		art_file = strrchr(file, '.');
+		if( art_file )
+		{
+			strcpy(art_file, ".jpg");
+			ret = access(file, R_OK);
+		}
+		if( ret != 0 )
+		{
+			art_file = strrchr(file, '/');
+			if( art_file )
+			{
+				memmove(art_file+2, art_file+1, file+MAXPATHLEN-art_file-2);
+				art_file[1] = '.';
+				ret = access(file, R_OK);
+			}
+		}
 	}
-	snprintf(file, sizeof(file), "%s", path);
-	art_file = strrchr(file, '.');
-	if( art_file )
-		strcpy(art_file, ".jpg");
-	if( access(file, R_OK) == 0 )
+	if( ret == 0 )
 	{
 		if( art_cache_exists(file, &art_file) )
 			goto existing_file;
@@ -333,14 +347,14 @@ found_file:
 	return NULL;
 }
 
-sqlite_int64
+int64_t
 find_album_art(const char *path, const char *image_data, int image_size)
 {
 	char *album_art = NULL;
 	char *sql;
 	char **result;
 	int cols, rows;
-	sqlite_int64 ret = 0;
+	int64_t ret = 0;
 
 	if( (image_size && (album_art = check_embedded_art(path, image_data, image_size))) ||
 	    (album_art = check_for_album_file(path)) )
