@@ -26,13 +26,18 @@
  * The resize functions come from the resize_image project, at http://www.golac.fr/Image-Resizer
  */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <setjmp.h>
 #include <jpeglib.h>
+#ifdef HAVE_MACHINE_ENDIAN_H
+#include <machine/endian.h>
+#else
 #include <endian.h>
+#endif
 
 #include "upnpreplyparse.h"
 #include "image_utils.h"
@@ -367,7 +372,7 @@ image_get_jpeg_date_xmp(const char * path, char ** date)
 			if( nread < 1 )
 				break;
 
-			ParseNameValue(data, offset, &xml);
+			ParseNameValue(data, offset, &xml, 0);
 			exif = GetValueFromNameValueList(&xml, "DateTimeOriginal");
 			if( !exif )
 			{
@@ -393,8 +398,7 @@ image_get_jpeg_date_xmp(const char * path, char ** date)
 		}
 	}
 	fclose(img);
-	if( data )
-		free(data);
+	free(data);
 	return ret;
 }
 
@@ -475,8 +479,7 @@ image_new_from_jpeg(const char * path, int is_file, const char * buf, int size,
 			fclose(file);
 		if( vimage )
 		{
-			if( vimage->buf )
-				free(vimage->buf);
+			free(vimage->buf);
 			free(vimage);
 		}
 		return NULL;
