@@ -15,50 +15,35 @@
  * You should have received a copy of the GNU General Public License
  * along with MiniDLNA. If not, see <http://www.gnu.org/licenses/>.
  */
+#include "config.h"
+
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
-#include <sys/stat.h>
-
+#include <libgen.h>
 #include <unistd.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/param.h>
 #include <fcntl.h>
 
 #include <libexif/exif-loader.h>
-#include "image_utils.h"
 #include <jpeglib.h>
 #include <setjmp.h>
-#include <avutil.h>
-#include <avcodec.h>
-#include <avformat.h>
-#include "tagutils/tagutils.h"
+#include "libav.h"
 
 #include "upnpglobalvars.h"
+#include "tagutils/tagutils.h"
+#include "image_utils.h"
 #include "upnpreplyparse.h"
+#include "tivo_utils.h"
 #include "metadata.h"
 #include "albumart.h"
 #include "utils.h"
 #include "sql.h"
 #include "log.h"
 
-#ifndef FF_PROFILE_H264_BASELINE
-#define FF_PROFILE_H264_BASELINE 66
-#endif
-#ifndef FF_PROFILE_H264_MAIN
-#define FF_PROFILE_H264_MAIN 77
-#endif
-#ifndef FF_PROFILE_H264_HIGH
-#define FF_PROFILE_H264_HIGH 100
-#endif
-#define FF_PROFILE_SKIP -100
-
-#if LIBAVCODEC_VERSION_MAJOR < 53
-#define AVMEDIA_TYPE_AUDIO CODEC_TYPE_AUDIO
-#define AVMEDIA_TYPE_VIDEO CODEC_TYPE_VIDEO
-#endif
-
 #define FLAG_TITLE	0x00000001
 #define FLAG_ARTIST	0x00000002
 #define FLAG_ALBUM	0x00000004
@@ -91,21 +76,59 @@ enum audio_profiles {
 	PROFILE_AUDIO_AMR
 };
 
+static inline int
+lav_open(AVFormatContext **ctx, const char *filename)
+{
+	int ret;
+#if LIBAVFORMAT_VERSION_INT >= ((53<<16)+(17<<8)+0)
+	ret = avformat_open_input(ctx, filename, NULL, NULL);
+	if (ret == 0)
+		avformat_find_stream_info(*ctx, NULL);
+#else
+	ret = av_open_input_file(ctx, filename, NULL, 0, NULL);
+	if (ret == 0)
+		av_find_stream_info(*ctx);
+#endif
+	return ret;
+}
+
+static inline void
+lav_close(AVFormatContext *ctx)
+{
+#if LIBAVFORMAT_VERSION_INT >= ((53<<16)+(17<<8)+0)
+	avformat_close_input(&ctx);
+#else
+	av_close_input_file(ctx);
+#endif
+}
+
+#if LIBAVFORMAT_VERSION_INT >= ((52<<16)+(31<<8)+0)
+# if LIBAVUTIL_VERSION_INT < ((51<<16)+(5<<8)+0) && !defined(FF_API_OLD_METADATA2)
+#define AV_DICT_IGNORE_SUFFIX AV_METADATA_IGNORE_SUFFIX
+#define av_dict_get av_metadata_get
+typedef AVMetadataTag AVDictionaryEntry;
+# endif
+#endif
+
 /* This function shamelessly copied from libdlna */
 #define MPEG_TS_SYNC_CODE 0x47
 #define MPEG_TS_PACKET_LENGTH 188
 #define MPEG_TS_PACKET_LENGTH_DLNA 192 /* prepends 4 bytes to TS packet */
 int
-dlna_timestamp_is_present(const char * filename, int * raw_packet_size)
+dlna_timestamp_is_present(const char *filename, int *raw_packet_size)
 {
 	unsigned char buffer[3*MPEG_TS_PACKET_LENGTH_DLNA];
 	int fd, i;
 
 	/* read file header */
 	fd = open(filename, O_RDONLY);
-	read(fd, buffer, MPEG_TS_PACKET_LENGTH_DLNA*3);
+	if( fd < 0 )
+		return 0;
+	i = read(fd, buffer, MPEG_TS_PACKET_LENGTH_DLNA*3);
 	close(fd);
-	for( i=0; i < MPEG_TS_PACKET_LENGTH_DLNA; i++ )
+	if( i < 0 )
+		return 0;
+	for( i = 0; i < MPEG_TS_PACKET_LENGTH_DLNA; i++ )
 	{
 		if( buffer[i] == MPEG_TS_SYNC_CODE )
 		{
@@ -131,27 +154,10 @@ dlna_timestamp_is_present(const char * filename, int * raw_packet_size)
 	return 0;
 }
 
-#ifdef TIVO_SUPPORT
-int
-is_tivo_file(const char * path)
-{
-	unsigned char buf[5];
-	unsigned char hdr[5] = { 'T','i','V','o','\0' };
-	int fd;
-
-	/* read file header */
-	fd = open(path, O_RDONLY);
-	read(fd, buf, 5);
-	close(fd);
-
-	return( !memcmp(buf, hdr, 5) );
-}
-#endif
-
 void
-check_for_captions(const char * path, sqlite_int64 detailID)
+check_for_captions(const char *path, int64_t detailID)
 {
-	char *file = malloc(PATH_MAX);
+	char *file = malloc(MAXPATHLEN);
 	char *id = NULL;
 
 	sprintf(file, "%s", path);
@@ -160,16 +166,16 @@ check_for_captions(const char * path, sqlite_int64 detailID)
 	/* If we weren't given a detail ID, look for one. */
 	if( !detailID )
 	{
-		id = sql_get_text_field(db, "SELECT ID from DETAILS where PATH glob '%q.*'"
-		                            " and MIME glob 'video/*' limit 1", file);
+		id = sql_get_text_field(db, "SELECT ID from DETAILS where (PATH > '%q.' and PATH <= '%q.z')"
+		                            " and MIME glob 'video/*' limit 1", file, file);
 		if( id )
 		{
-			//DEBUG DPRINTF(E_DEBUG, L_METADATA, "New file %s looks like a caption file.\n", path);
+			//DPRINTF(E_MAXDEBUG, L_METADATA, "New file %s looks like a caption file.\n", path);
 			detailID = strtoll(id, NULL, 10);
 		}
 		else
 		{
-			//DPRINTF(E_DEBUG, L_METADATA, "No file found for caption %s.\n", path);
+			//DPRINTF(E_MAXDEBUG, L_METADATA, "No file found for caption %s.\n", path);
 			goto no_source_video;
 		}
 	}
@@ -189,7 +195,7 @@ no_source_video:
 }
 
 void
-parse_nfo(const char * path, metadata_t * m)
+parse_nfo(const char *path, metadata_t *m)
 {
 	FILE *nfo;
 	char buf[65536];
@@ -210,7 +216,7 @@ parse_nfo(const char * path, metadata_t * m)
 		return;
 	nread = fread(&buf, 1, sizeof(buf), nfo);
 	
-	ParseNameValue(buf, nread, &xml);
+	ParseNameValue(buf, nread, &xml, 0);
 
 	//printf("\ttype: %s\n", GetValueFromNameValueList(&xml, "rootElement"));
 	val = GetValueFromNameValueList(&xml, "title");
@@ -218,7 +224,7 @@ parse_nfo(const char * path, metadata_t * m)
 	{
 		val2 = GetValueFromNameValueList(&xml, "episodetitle");
 		if( val2 )
-			asprintf(&m->title, "%s - %s", val, val2);
+			xasprintf(&m->title, "%s - %s", val, val2);
 		else
 			m->title = strdup(val);
 	}
@@ -233,14 +239,24 @@ parse_nfo(const char * path, metadata_t * m)
 
 	val = GetValueFromNameValueList(&xml, "genre");
 	if( val )
+	{
+		free(m->genre);
 		m->genre = strdup(val);
+	}
+
+	val = GetValueFromNameValueList(&xml, "mime");
+	if( val )
+	{
+		free(m->mime);
+		m->mime = strdup(val);
+	}
 
 	ClearNameValueList(&xml);
 	fclose(nfo);
 }
 
 void
-free_metadata(metadata_t * m, uint32_t flags)
+free_metadata(metadata_t *m, uint32_t flags)
 {
 	if( flags & FLAG_TITLE )
 		free(m->title);
@@ -276,8 +292,8 @@ free_metadata(metadata_t * m, uint32_t flags)
 		free(m->rotation);
 }
 
-sqlite_int64
-GetFolderMetadata(const char * name, const char * path, const char * artist, const char * genre, sqlite3_int64 album_art)
+int64_t
+GetFolderMetadata(const char *name, const char *path, const char *artist, const char *genre, int64_t album_art)
 {
 	int ret;
 
@@ -294,16 +310,16 @@ GetFolderMetadata(const char * name, const char * path, const char * artist, con
 	return ret;
 }
 
-sqlite_int64
-GetAudioMetadata(const char * path, char * name)
+int64_t
+GetAudioMetadata(const char *path, char *name)
 {
 	char type[4];
 	static char lang[6] = { '\0' };
 	struct stat file;
-	sqlite_int64 ret;
+	int64_t ret;
 	char *esc_tag;
 	int i;
-	sqlite_int64 album_art = 0;
+	int64_t album_art = 0;
 	struct song_metadata song;
 	metadata_t m;
 	uint32_t free_flags = FLAG_MIME|FLAG_DURATION|FLAG_DLNA_PN|FLAG_DATE;
@@ -379,8 +395,8 @@ GetAudioMetadata(const char * path, char * name)
 	if( song.dlna_pn )
 		m.dlna_pn = strdup(song.dlna_pn);
 	if( song.year )
-		asprintf(&m.date, "%04d-01-01", song.year);
-	asprintf(&m.duration, "%d:%02d:%02d.%03d",
+		xasprintf(&m.date, "%04d-01-01", song.year);
+	xasprintf(&m.duration, "%d:%02d:%02d.%03d",
 	                      (song.song_length/3600000),
 	                      (song.song_length/60000%60),
 	                      (song.song_length/1000%60),
@@ -499,8 +515,8 @@ libjpeg_error_handler(j_common_ptr cinfo)
 	return;
 }
 
-sqlite_int64
-GetImageMetadata(const char * path, char * name)
+int64_t
+GetImageMetadata(const char *path, char *name)
 {
 	ExifData *ed;
 	ExifEntry *e = NULL;
@@ -512,8 +528,8 @@ GetImageMetadata(const char * path, char * name)
 	char make[32], model[64] = {'\0'};
 	char b[1024];
 	struct stat file;
-	sqlite_int64 ret;
-	image_s * imsrc;
+	int64_t ret;
+	image_s *imsrc;
 	metadata_t m;
 	uint32_t free_flags = 0xFFFFFFFF;
 	memset(&m, '\0', sizeof(metadata_t));
@@ -590,10 +606,7 @@ GetImageMetadata(const char * path, char * name)
 			break;
 		}
 		if( rotate )
-		{
-			if( asprintf(&m.rotation, "%d", rotate) < 0 )
-				m.rotation = NULL;
-		}
+			xasprintf(&m.rotation, "%d", rotate);
 	}
 
 	if( ed->size )
@@ -649,9 +662,9 @@ no_exifdata:
 		m.dlna_pn = strdup("JPEG_SM");
 	else if( width <= 1024 && height <= 768 )
 		m.dlna_pn = strdup("JPEG_MED");
-	else if( (width <= 4096 && height <= 4096) || !(GETFLAG(DLNA_STRICT_MASK)) )
+	else if( (width <= 4096 && height <= 4096) || !GETFLAG(DLNA_STRICT_MASK) )
 		m.dlna_pn = strdup("JPEG_LRG");
-	asprintf(&m.resolution, "%dx%d", width, height);
+	xasprintf(&m.resolution, "%dx%d", width, height);
 
 	ret = sql_exec(db, "INSERT into DETAILS"
 	                   " (PATH, TITLE, SIZE, TIMESTAMP, DATE, RESOLUTION,"
@@ -674,8 +687,8 @@ no_exifdata:
 	return ret;
 }
 
-sqlite_int64
-GetVideoMetadata(const char * path, char * name)
+int64_t
+GetVideoMetadata(const char *path, char *name)
 {
 	struct stat file;
 	int ret, i;
@@ -685,11 +698,12 @@ GetVideoMetadata(const char * path, char * name)
 	int audio_stream = -1, video_stream = -1;
 	enum audio_profiles audio_profile = PROFILE_AUDIO_UNKNOWN;
 	char fourcc[4];
-	sqlite_int64 album_art = 0;
-	char nfo[PATH_MAX], *ext;
+	int64_t album_art = 0;
+	char nfo[MAXPATHLEN], *ext;
 	struct song_metadata video;
 	metadata_t m;
 	uint32_t free_flags = 0xFFFFFFFF;
+	char *path_cpy, *basepath;
 
 	memset(&m, '\0', sizeof(m));
 	memset(&video, '\0', sizeof(video));
@@ -700,17 +714,14 @@ GetVideoMetadata(const char * path, char * name)
 	strip_ext(name);
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, " * size: %jd\n", file.st_size);
 
-	av_register_all();
-	#if LIBAVFORMAT_VERSION_INT >= ((53<<16)+(2<<8)+0)
-	if( avformat_open_input(&ctx, path, NULL, NULL) != 0 )
-	#else
-	if( av_open_input_file(&ctx, path, NULL, 0, NULL) != 0 )
-	#endif
+	ret = lav_open(&ctx, path);
+	if( ret != 0 )
 	{
-		DPRINTF(E_WARN, L_METADATA, "Opening %s failed!\n", path);
+		char err[128];
+		av_strerror(ret, err, sizeof(err));
+		DPRINTF(E_WARN, L_METADATA, "Opening %s failed! [%s]\n", path, err);
 		return 0;
 	}
-	av_find_stream_info(ctx);
 	//dump_format(ctx, 0, NULL, 0);
 	for( i=0; i<ctx->nb_streams; i++)
 	{
@@ -729,33 +740,18 @@ GetVideoMetadata(const char * path, char * name)
 			continue;
 		}
 	}
-	/* This must not be a video file. */
+	path_cpy = strdup(path);
+	basepath = basename(path_cpy);
 	if( !vc )
 	{
-		av_close_input_file(ctx);
+		/* This must not be a video file. */
+		lav_close(ctx);
 		if( !is_audio(path) )
-			DPRINTF(E_DEBUG, L_METADATA, "File %s does not contain a video stream.\n", basename(path));
+			DPRINTF(E_DEBUG, L_METADATA, "File %s does not contain a video stream.\n", basepath);
+		free(path_cpy);
 		return 0;
 	}
 
-	strcpy(nfo, path);
-	ext = strrchr(nfo, '.');
-	if( ext )
-	{
-		strcpy(ext+1, "nfo");
-		if( access(nfo, F_OK) == 0 )
-		{
-			parse_nfo(nfo, &m);
-		}
-	}
-
-	if( !m.date )
-	{
-		m.date = malloc(20);
-		modtime = localtime(&file.st_mtime);
-		strftime(m.date, 20, "%FT%T", modtime);
-	}
-
 	if( ac )
 	{
 		aac_object_type_t aac_type = AAC_INVALID;
@@ -837,37 +833,37 @@ GetVideoMetadata(const char * path, char * name)
 					DPRINTF(E_DEBUG, L_METADATA, "Unhandled audio codec [0x%X]\n", ac->codec_id);
 				break;
 		}
-		asprintf(&m.frequency, "%u", ac->sample_rate);
+		xasprintf(&m.frequency, "%u", ac->sample_rate);
 		#if LIBAVCODEC_VERSION_INT < (52<<16)
-		asprintf(&m.bps, "%u", ac->bits_per_sample);
+		xasprintf(&m.bps, "%u", ac->bits_per_sample);
 		#else
-		asprintf(&m.bps, "%u", ac->bits_per_coded_sample);
+		xasprintf(&m.bps, "%u", ac->bits_per_coded_sample);
 		#endif
-		asprintf(&m.channels, "%u", ac->channels);
+		xasprintf(&m.channels, "%u", ac->channels);
 	}
 	if( vc )
 	{
 		int off;
 		int duration, hours, min, sec, ms;
 		ts_timestamp_t ts_timestamp = NONE;
-		DPRINTF(E_DEBUG, L_METADATA, "Container: '%s' [%s]\n", ctx->iformat->name, basename(path));
-		asprintf(&m.resolution, "%dx%d", vc->width, vc->height);
+		DPRINTF(E_DEBUG, L_METADATA, "Container: '%s' [%s]\n", ctx->iformat->name, basepath);
+		xasprintf(&m.resolution, "%dx%d", vc->width, vc->height);
 		if( ctx->bit_rate > 8 )
-			asprintf(&m.bitrate, "%u", ctx->bit_rate / 8);
+			xasprintf(&m.bitrate, "%u", ctx->bit_rate / 8);
 		if( ctx->duration > 0 ) {
 			duration = (int)(ctx->duration / AV_TIME_BASE);
 			hours = (int)(duration / 3600);
 			min = (int)(duration / 60 % 60);
 			sec = (int)(duration % 60);
 			ms = (int)(ctx->duration / (AV_TIME_BASE/1000) % 1000);
-			asprintf(&m.duration, "%d:%02d:%02d.%03d", hours, min, sec, ms);
+			xasprintf(&m.duration, "%d:%02d:%02d.%03d", hours, min, sec, ms);
 		}
 
 		/* NOTE: The DLNA spec only provides for ASF (WMV), TS, PS, and MP4 containers.
 		 * Skip DLNA parsing for everything else. */
 		if( strcmp(ctx->iformat->name, "avi") == 0 )
 		{
-			asprintf(&m.mime, "video/x-msvideo");
+			xasprintf(&m.mime, "video/x-msvideo");
 			if( vc->codec_id == CODEC_ID_MPEG4 )
 			{
         			fourcc[0] = vc->codec_tag     & 0xff;
@@ -877,16 +873,16 @@ GetVideoMetadata(const char * path, char * name)
 				if( memcmp(fourcc, "XVID", 4) == 0 ||
 				    memcmp(fourcc, "DX50", 4) == 0 ||
 				    memcmp(fourcc, "DIVX", 4) == 0 )
-					asprintf(&m.creator, "DiVX");
+					xasprintf(&m.creator, "DiVX");
 			}
 		}
 		else if( strcmp(ctx->iformat->name, "mov,mp4,m4a,3gp,3g2,mj2") == 0 &&
 		         ends_with(path, ".mov") )
-			asprintf(&m.mime, "video/quicktime");
+			xasprintf(&m.mime, "video/quicktime");
 		else if( strncmp(ctx->iformat->name, "matroska", 8) == 0 )
-			asprintf(&m.mime, "video/x-matroska");
+			xasprintf(&m.mime, "video/x-matroska");
 		else if( strcmp(ctx->iformat->name, "flv") == 0 )
-			asprintf(&m.mime, "video/x-flv");
+			xasprintf(&m.mime, "video/x-flv");
 		if( m.mime )
 			goto video_no_dlna;
 
@@ -900,7 +896,7 @@ GetVideoMetadata(const char * path, char * name)
 					{
 						m.dlna_pn = strdup("MPEG1");
 					}
-					asprintf(&m.mime, "video/mpeg");
+					xasprintf(&m.mime, "video/mpeg");
 				}
 				break;
 			case CODEC_ID_MPEG2VIDEO:
@@ -911,7 +907,7 @@ GetVideoMetadata(const char * path, char * name)
 					int raw_packet_size;
 					int dlna_ts_present = dlna_timestamp_is_present(path, &raw_packet_size);
 					DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is %s MPEG2 TS packet size %d\n",
-						video_stream, basename(path), m.resolution, raw_packet_size);
+						video_stream, basepath, m.resolution, raw_packet_size);
 					off += sprintf(m.dlna_pn+off, "TS_");
 					if( (vc->width  >= 1280) &&
 					    (vc->height >= 720) )
@@ -937,21 +933,21 @@ GetVideoMetadata(const char * path, char * name)
 					else if( raw_packet_size != MPEG_TS_PACKET_LENGTH )
 					{
 						DPRINTF(E_DEBUG, L_METADATA, "Unsupported DLNA TS packet size [%d] (%s)\n",
-							raw_packet_size, basename(path));
+							raw_packet_size, basepath);
 						free(m.dlna_pn);
 						m.dlna_pn = NULL;
 					}
 					switch( ts_timestamp )
 					{
 						case NONE:
-							asprintf(&m.mime, "video/mpeg");
+							xasprintf(&m.mime, "video/mpeg");
 							if( m.dlna_pn )
 								off += sprintf(m.dlna_pn+off, "_ISO");
 							break;
 						case VALID:
 							off += sprintf(m.dlna_pn+off, "_T");
 						case EMPTY:
-							asprintf(&m.mime, "video/vnd.dlna.mpeg-tts");
+							xasprintf(&m.mime, "video/vnd.dlna.mpeg-tts");
 						default:
 							break;
 					}
@@ -959,19 +955,19 @@ GetVideoMetadata(const char * path, char * name)
 				else if( strcmp(ctx->iformat->name, "mpeg") == 0 )
 				{
 					DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is %s MPEG2 PS\n",
-						video_stream, basename(path), m.resolution);
+						video_stream, basepath, m.resolution);
 					off += sprintf(m.dlna_pn+off, "PS_");
 					if( (vc->height == 576) ||
 					    (vc->height == 288) )
 						off += sprintf(m.dlna_pn+off, "PAL");
 					else
 						off += sprintf(m.dlna_pn+off, "NTSC");
-					asprintf(&m.mime, "video/mpeg");
+					xasprintf(&m.mime, "video/mpeg");
 				}
 				else
 				{
 					DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s [%s] is %s non-DLNA MPEG2\n",
-						video_stream, basename(path), ctx->iformat->name, m.resolution);
+						video_stream, basepath, ctx->iformat->name, m.resolution);
 					free(m.dlna_pn);
 					m.dlna_pn = NULL;
 				}
@@ -994,8 +990,11 @@ GetVideoMetadata(const char * path, char * name)
 						          vc->height * vc->sample_aspect_ratio.den,
 						          1024*1024);
 					}
-					fps = ctx->streams[video_stream]->r_frame_rate.num / ctx->streams[video_stream]->r_frame_rate.den;
-					interlaced = (ctx->streams[video_stream]->r_frame_rate.num / vc->time_base.den);
+					if (ctx->streams[video_stream]->r_frame_rate.den)
+						fps = ctx->streams[video_stream]->r_frame_rate.num / ctx->streams[video_stream]->r_frame_rate.den;
+					else
+						fps = 0;
+					interlaced = vc->time_base.den ? (ctx->streams[video_stream]->r_frame_rate.num / vc->time_base.den) : 0;
 					if( ((((vc->width == 1920 || vc->width == 1440) && vc->height == 1080) ||
 					      (vc->width == 720 && vc->height == 480)) && fps == 59 && interlaced) ||
 					    ((vc->width == 1280 && vc->height == 720) && fps == 59 && !interlaced) )
@@ -1023,6 +1022,7 @@ GetVideoMetadata(const char * path, char * name)
 					switch( vc->profile )
 					{
 						case FF_PROFILE_H264_BASELINE:
+						case FF_PROFILE_H264_CONSTRAINED_BASELINE:
 							off += sprintf(m.dlna_pn+off, "BL_");
 							if( vc->width  <= 352 &&
 							    vc->height <= 288 &&
@@ -1045,10 +1045,11 @@ GetVideoMetadata(const char * path, char * name)
 						case FF_PROFILE_H264_MAIN:
 							off += sprintf(m.dlna_pn+off, "MP_");
 							if( vc->profile != FF_PROFILE_H264_BASELINE &&
+							    vc->profile != FF_PROFILE_H264_CONSTRAINED_BASELINE &&
 							    vc->profile != FF_PROFILE_H264_MAIN )
 							{
 								DPRINTF(E_DEBUG, L_METADATA, "Unknown AVC profile %d; assuming MP. [%s]\n",
-									vc->profile, basename(path));
+									vc->profile, basepath);
 							}
 							if( vc->width  <= 720 &&
 							    vc->height <= 576 &&
@@ -1065,7 +1066,7 @@ GetVideoMetadata(const char * path, char * name)
 							else
 							{
 								DPRINTF(E_DEBUG, L_METADATA, "Unsupported h.264 video profile! [%s, %dx%d, %dbps : %s]\n",
-									m.dlna_pn, vc->width, vc->height, vc->bit_rate, basename(path));
+									m.dlna_pn, vc->width, vc->height, vc->bit_rate, basepath);
 								free(m.dlna_pn);
 								m.dlna_pn = NULL;
 							}
@@ -1082,7 +1083,7 @@ GetVideoMetadata(const char * path, char * name)
 							else
 							{
 								DPRINTF(E_DEBUG, L_METADATA, "Unsupported h.264 HP video profile! [%dbps, %d audio : %s]\n",
-									vc->bit_rate, audio_profile, basename(path));
+									vc->bit_rate, audio_profile, basepath);
 								free(m.dlna_pn);
 								m.dlna_pn = NULL;
 							}
@@ -1106,7 +1107,7 @@ GetVideoMetadata(const char * path, char * name)
 							break;
 						default:
 							DPRINTF(E_DEBUG, L_METADATA, "No DLNA profile found for %s file [%s]\n",
-								m.dlna_pn, basename(path));
+								m.dlna_pn, basepath);
 							free(m.dlna_pn);
 							m.dlna_pn = NULL;
 							break;
@@ -1124,7 +1125,7 @@ GetVideoMetadata(const char * path, char * name)
 					else if( raw_packet_size != MPEG_TS_PACKET_LENGTH )
 					{
 						DPRINTF(E_DEBUG, L_METADATA, "Unsupported DLNA TS packet size [%d] (%s)\n",
-							raw_packet_size, basename(path));
+							raw_packet_size, basepath);
 						free(m.dlna_pn);
 						m.dlna_pn = NULL;
 					}
@@ -1137,7 +1138,7 @@ GetVideoMetadata(const char * path, char * name)
 						case VALID:
 							off += sprintf(m.dlna_pn+off, "_T");
 						case EMPTY:
-							asprintf(&m.mime, "video/vnd.dlna.mpeg-tts");
+							xasprintf(&m.mime, "video/vnd.dlna.mpeg-tts");
 						default:
 							break;
 					}
@@ -1148,6 +1149,7 @@ GetVideoMetadata(const char * path, char * name)
 
 					switch( vc->profile ) {
 					case FF_PROFILE_H264_BASELINE:
+					case FF_PROFILE_H264_CONSTRAINED_BASELINE:
 						if( vc->width  <= 352 &&
 						    vc->height <= 288 )
 						{
@@ -1252,7 +1254,7 @@ GetVideoMetadata(const char * path, char * name)
 						if( strlen(m.dlna_pn) <= 11 )
 						{
 							DPRINTF(E_DEBUG, L_METADATA, "No DLNA profile found for %s file %s\n",
-								m.dlna_pn, basename(path));
+								m.dlna_pn, basepath);
 							free(m.dlna_pn);
 							m.dlna_pn = NULL;
 						}
@@ -1268,7 +1270,7 @@ GetVideoMetadata(const char * path, char * name)
 						break;
 					default:
 						DPRINTF(E_DEBUG, L_METADATA, "AVC profile [%d] not recognized for file %s\n",
-							vc->profile, basename(path));
+							vc->profile, basepath);
 						free(m.dlna_pn);
 						m.dlna_pn = NULL;
 						break;
@@ -1279,7 +1281,7 @@ GetVideoMetadata(const char * path, char * name)
 					free(m.dlna_pn);
 					m.dlna_pn = NULL;
 				}
-				DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is h.264\n", video_stream, basename(path));
+				DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is h.264\n", video_stream, basepath);
 				break;
 			case CODEC_ID_MPEG4:
         			fourcc[0] = vc->codec_tag     & 0xff;
@@ -1287,7 +1289,7 @@ GetVideoMetadata(const char * path, char * name)
 			        fourcc[2] = vc->codec_tag>>16 & 0xff;
 			        fourcc[3] = vc->codec_tag>>24 & 0xff;
 				DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is MPEG4 [%c%c%c%c/0x%X]\n",
-					video_stream, basename(path),
+					video_stream, basepath,
 					isprint(fourcc[0]) ? fourcc[0] : '_',
 					isprint(fourcc[1]) ? fourcc[1] : '_',
 					isprint(fourcc[2]) ? fourcc[2] : '_',
@@ -1301,7 +1303,7 @@ GetVideoMetadata(const char * path, char * name)
 
 					if( ends_with(path, ".3gp") )
 					{
-						asprintf(&m.mime, "video/3gpp");
+						xasprintf(&m.mime, "video/3gpp");
 						switch( audio_profile )
 						{
 							case PROFILE_AUDIO_AAC:
@@ -1312,7 +1314,7 @@ GetVideoMetadata(const char * path, char * name)
 								break;
 							default:
 								DPRINTF(E_DEBUG, L_METADATA, "No DLNA profile found for MPEG4-P2 3GP/0x%X file %s\n",
-								        ac->codec_id, basename(path));
+								        ac->codec_id, basepath);
 								free(m.dlna_pn);
 								m.dlna_pn = NULL;
 								break;
@@ -1361,8 +1363,8 @@ GetVideoMetadata(const char * path, char * name)
 				}
 				m.dlna_pn = malloc(64);
 				off = sprintf(m.dlna_pn, "WMV");
-				DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is VC1\n", video_stream, basename(path));
-				asprintf(&m.mime, "video/x-ms-wmv");
+				DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is VC1\n", video_stream, basepath);
+				xasprintf(&m.mime, "video/x-ms-wmv");
 				if( (vc->width  <= 176) &&
 				    (vc->height <= 144) &&
 				    (vc->level == 0) )
@@ -1378,7 +1380,7 @@ GetVideoMetadata(const char * path, char * name)
 							break;
 						default:
 							DPRINTF(E_DEBUG, L_METADATA, "No DLNA profile found for WMVSPLL/0x%X file %s\n",
-								audio_profile, basename(path));
+								audio_profile, basepath);
 							free(m.dlna_pn);
 							m.dlna_pn = NULL;
 							break;
@@ -1400,7 +1402,7 @@ GetVideoMetadata(const char * path, char * name)
 							break;
 						default:
 							DPRINTF(E_DEBUG, L_METADATA, "No DLNA profile found for WMVSPML/0x%X file %s\n",
-								audio_profile, basename(path));
+								audio_profile, basepath);
 							free(m.dlna_pn);
 							m.dlna_pn = NULL;
 							break;
@@ -1424,7 +1426,7 @@ GetVideoMetadata(const char * path, char * name)
 							break;
 						default:
 							DPRINTF(E_DEBUG, L_METADATA, "No DLNA profile found for WMVMED/0x%X file %s\n",
-								audio_profile, basename(path));
+								audio_profile, basepath);
 							free(m.dlna_pn);
 							m.dlna_pn = NULL;
 							break;
@@ -1445,7 +1447,7 @@ GetVideoMetadata(const char * path, char * name)
 							break;
 						default:
 							DPRINTF(E_DEBUG, L_METADATA, "No DLNA profile found for WMVHIGH/0x%X file %s\n",
-								audio_profile, basename(path));
+								audio_profile, basepath);
 							free(m.dlna_pn);
 							m.dlna_pn = NULL;
 							break;
@@ -1453,33 +1455,13 @@ GetVideoMetadata(const char * path, char * name)
 				}
 				break;
 			case CODEC_ID_MSMPEG4V3:
-				asprintf(&m.mime, "video/x-msvideo");
+				xasprintf(&m.mime, "video/x-msvideo");
 			default:
 				DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is %s [type %d]\n",
-					video_stream, basename(path), m.resolution, vc->codec_id);
+					video_stream, basepath, m.resolution, vc->codec_id);
 				break;
 		}
 	}
-	if( !m.mime )
-	{
-		if( strcmp(ctx->iformat->name, "avi") == 0 )
-			asprintf(&m.mime, "video/x-msvideo");
-		else if( strncmp(ctx->iformat->name, "mpeg", 4) == 0 )
-			asprintf(&m.mime, "video/mpeg");
-		else if( strcmp(ctx->iformat->name, "asf") == 0 )
-			asprintf(&m.mime, "video/x-ms-wmv");
-		else if( strcmp(ctx->iformat->name, "mov,mp4,m4a,3gp,3g2,mj2") == 0 )
-			if( ends_with(path, ".mov") )
-				asprintf(&m.mime, "video/quicktime");
-			else
-				asprintf(&m.mime, "video/mp4");
-		else if( strncmp(ctx->iformat->name, "matroska", 8) == 0 )
-			asprintf(&m.mime, "video/x-matroska");
-		else if( strcmp(ctx->iformat->name, "flv") == 0 )
-			asprintf(&m.mime, "video/x-flv");
-		else
-			DPRINTF(E_WARN, L_METADATA, "%s: Unhandled format: %s\n", path, ctx->iformat->name);
-	}
 
 	if( strcmp(ctx->iformat->name, "asf") == 0 )
 	{
@@ -1514,10 +1496,10 @@ GetVideoMetadata(const char * path, char * name)
 	{
 		if( ctx->metadata )
 		{
-			AVMetadataTag *tag = NULL;
+			AVDictionaryEntry *tag = NULL;
 
 			//DEBUG DPRINTF(E_DEBUG, L_METADATA, "Metadata:\n");
-			while( (tag = av_metadata_get(ctx->metadata, "", tag, AV_METADATA_IGNORE_SUFFIX)) )
+			while( (tag = av_dict_get(ctx->metadata, "", tag, AV_DICT_IGNORE_SUFFIX)) )
 			{
 				//DEBUG DPRINTF(E_DEBUG, L_METADATA, "  %-16s: %s\n", tag->key, tag->value);
 				if( strcmp(tag->key, "title") == 0 )
@@ -1534,7 +1516,6 @@ GetVideoMetadata(const char * path, char * name)
 	#endif
 	#endif
 video_no_dlna:
-	av_close_input_file(ctx);
 
 #ifdef TIVO_SUPPORT
 	if( ends_with(path, ".TiVo") && is_tivo_file(path) )
@@ -1544,10 +1525,51 @@ video_no_dlna:
 			free(m.dlna_pn);
 			m.dlna_pn = NULL;
 		}
-		m.mime = realloc(m.mime, 18);
+		m.mime = realloc(m.mime, 21);
 		strcpy(m.mime, "video/x-tivo-mpeg");
 	}
 #endif
+
+	strcpy(nfo, path);
+	ext = strrchr(nfo, '.');
+	if( ext )
+	{
+		strcpy(ext+1, "nfo");
+		if( access(nfo, F_OK) == 0 )
+		{
+			parse_nfo(nfo, &m);
+		}
+	}
+
+	if( !m.mime )
+	{
+		if( strcmp(ctx->iformat->name, "avi") == 0 )
+			xasprintf(&m.mime, "video/x-msvideo");
+		else if( strncmp(ctx->iformat->name, "mpeg", 4) == 0 )
+			xasprintf(&m.mime, "video/mpeg");
+		else if( strcmp(ctx->iformat->name, "asf") == 0 )
+			xasprintf(&m.mime, "video/x-ms-wmv");
+		else if( strcmp(ctx->iformat->name, "mov,mp4,m4a,3gp,3g2,mj2") == 0 )
+			if( ends_with(path, ".mov") )
+				xasprintf(&m.mime, "video/quicktime");
+			else
+				xasprintf(&m.mime, "video/mp4");
+		else if( strncmp(ctx->iformat->name, "matroska", 8) == 0 )
+			xasprintf(&m.mime, "video/x-matroska");
+		else if( strcmp(ctx->iformat->name, "flv") == 0 )
+			xasprintf(&m.mime, "video/x-flv");
+		else
+			DPRINTF(E_WARN, L_METADATA, "%s: Unhandled format: %s\n", path, ctx->iformat->name);
+	}
+	lav_close(ctx);
+
+	if( !m.date )
+	{
+		m.date = malloc(20);
+		modtime = localtime(&file.st_mtime);
+		strftime(m.date, 20, "%FT%T", modtime);
+	}
+
 	if( !m.title )
 		m.title = strdup(name);
 
@@ -1574,6 +1596,7 @@ video_no_dlna:
 		check_for_captions(path, ret);
 	}
 	free_metadata(&m, free_flags);
+	free(path_cpy);
 
 	return ret;
 }
