@@ -29,6 +29,8 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,8 +42,6 @@
 #include <arpa/inet.h>
 #include <errno.h>
 
-#include "config.h"
-#include "upnpdescstrings.h"
 #include "minidlnapath.h"
 #include "upnphttp.h"
 #include "upnpglobalvars.h"
@@ -63,8 +63,7 @@ AddMulticastMembership(int s, in_addr_t ifaddr)
 
 	/* setting up imr structure */
 	imr.imr_multiaddr.s_addr = inet_addr(SSDP_MCAST_ADDR);
-	/*imr.imr_interface.s_addr = htonl(INADDR_ANY);*/
-	imr.imr_interface.s_addr = ifaddr;	/*inet_addr(ifaddr);*/
+	imr.imr_interface.s_addr = ifaddr;
 	
 	if (setsockopt(s, IPPROTO_IP, IP_ADD_MEMBERSHIP, (void *)&imr, sizeof(struct ip_mreq)) < 0)
 	{
@@ -78,32 +77,29 @@ AddMulticastMembership(int s, in_addr_t ifaddr)
 /* Open and configure the socket listening for 
  * SSDP udp packets sent on 239.255.255.250 port 1900 */
 int
-OpenAndConfSSDPReceiveSocket()
+OpenAndConfSSDPReceiveSocket(void)
 {
 	int s;
 	int i = 1;
 	struct sockaddr_in sockname;
 	
-	if( (s = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
+	s = socket(PF_INET, SOCK_DGRAM, 0);
+	if (s < 0)
 	{
 		DPRINTF(E_ERROR, L_SSDP, "socket(udp): %s\n", strerror(errno));
 		return -1;
 	}	
 
-	if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)) < 0)
-	{
+	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)) < 0)
 		DPRINTF(E_WARN, L_SSDP, "setsockopt(udp, SO_REUSEADDR): %s\n", strerror(errno));
-	}
 	
 	memset(&sockname, 0, sizeof(struct sockaddr_in));
 	sockname.sin_family = AF_INET;
 	sockname.sin_port = htons(SSDP_PORT);
 	/* NOTE : it seems it doesnt work when binding on the specific address */
-	/*sockname.sin_addr.s_addr = inet_addr(UPNP_MCAST_ADDR);*/
 	sockname.sin_addr.s_addr = htonl(INADDR_ANY);
-	/*sockname.sin_addr.s_addr = inet_addr(ifaddr);*/
 
-	if(bind(s, (struct sockaddr *)&sockname, sizeof(struct sockaddr_in)) < 0)
+	if (bind(s, (struct sockaddr *)&sockname, sizeof(struct sockaddr_in)) < 0)
 	{
 		DPRINTF(E_ERROR, L_SSDP, "bind(udp): %s\n", strerror(errno));
 		close(s);
@@ -111,10 +107,10 @@ OpenAndConfSSDPReceiveSocket()
 	}
 
 	i = n_lan_addr;
-	while(i>0)
+	while (i > 0)
 	{
 		i--;
-		if(AddMulticastMembership(s, lan_addr[i].addr.s_addr) < 0)
+		if (AddMulticastMembership(s, lan_addr[i].addr.s_addr) < 0)
 		{
 			DPRINTF(E_WARN, L_SSDP,
 			       "Failed to add multicast membership for address %s\n", 
@@ -127,7 +123,7 @@ OpenAndConfSSDPReceiveSocket()
 
 /* open the UDP socket used to send SSDP notifications to
  * the multicast group reserved for them */
-static int
+int
 OpenAndConfSSDPNotifySocket(in_addr_t addr)
 {
 	int s;
@@ -137,7 +133,8 @@ OpenAndConfSSDPNotifySocket(in_addr_t addr)
 	struct in_addr mc_if;
 	struct sockaddr_in sockname;
 	
-	if( (s = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
+	s = socket(PF_INET, SOCK_DGRAM, 0);
+	if (s < 0)
 	{
 		DPRINTF(E_ERROR, L_SSDP, "socket(udp_notify): %s\n", strerror(errno));
 		return -1;
@@ -145,14 +142,14 @@ OpenAndConfSSDPNotifySocket(in_addr_t addr)
 
 	mc_if.s_addr = addr;	/*inet_addr(addr);*/
 
-	if(setsockopt(s, IPPROTO_IP, IP_MULTICAST_LOOP, (char *)&loopchar, sizeof(loopchar)) < 0)
+	if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_LOOP, (char *)&loopchar, sizeof(loopchar)) < 0)
 	{
 		DPRINTF(E_ERROR, L_SSDP, "setsockopt(udp_notify, IP_MULTICAST_LOOP): %s\n", strerror(errno));
 		close(s);
 		return -1;
 	}
 
-	if(setsockopt(s, IPPROTO_IP, IP_MULTICAST_IF, (char *)&mc_if, sizeof(mc_if)) < 0)
+	if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_IF, (char *)&mc_if, sizeof(mc_if)) < 0)
 	{
 		DPRINTF(E_ERROR, L_SSDP, "setsockopt(udp_notify, IP_MULTICAST_IF): %s\n", strerror(errno));
 		close(s);
@@ -161,7 +158,7 @@ OpenAndConfSSDPNotifySocket(in_addr_t addr)
 
 	setsockopt(s, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));
 	
-	if(setsockopt(s, SOL_SOCKET, SO_BROADCAST, &bcast, sizeof(bcast)) < 0)
+	if (setsockopt(s, SOL_SOCKET, SO_BROADCAST, &bcast, sizeof(bcast)) < 0)
 	{
 		DPRINTF(E_ERROR, L_SSDP, "setsockopt(udp_notify, SO_BROADCAST): %s\n", strerror(errno));
 		close(s);
@@ -182,47 +179,6 @@ OpenAndConfSSDPNotifySocket(in_addr_t addr)
 	return s;
 }
 
-int
-OpenAndConfSSDPNotifySockets(int * sockets)
-{
-	int i, j;
-	for(i=0; i<n_lan_addr; i++)
-	{
-		sockets[i] = OpenAndConfSSDPNotifySocket(lan_addr[i].addr.s_addr);
-		if(sockets[i] < 0)
-		{
-			for(j=0; j<i; j++)
-			{
-				close(sockets[j]);
-				sockets[j] = -1;
-			}
-			return -1;
-		}
-	}
-	return 0;
-}
-
-/*
- * response from a LiveBox (Wanadoo)
-HTTP/1.1 200 OK
-CACHE-CONTROL: max-age=1800
-DATE: Thu, 01 Jan 1970 04:03:23 GMT
-EXT:
-LOCATION: http://192.168.0.1:49152/gatedesc.xml
-SERVER: Linux/2.4.17, UPnP/1.0, Intel SDK for UPnP devices /1.2
-ST: upnp:rootdevice
-USN: uuid:75802409-bccb-40e7-8e6c-fa095ecce13e::upnp:rootdevice
-
- * response from a Linksys 802.11b :
-HTTP/1.1 200 OK
-Cache-Control:max-age=120
-Location:http://192.168.5.1:5678/rootDesc.xml
-Server:NT/5.0 UPnP/1.0
-ST:upnp:rootdevice
-USN:uuid:upnp-InternetGatewayDevice-1_0-0090a2777777::upnp:rootdevice
-EXT:
- */
-
 static const char * const known_service_types[] =
 {
 	uuidvalue,
@@ -247,11 +203,14 @@ _usleep(long usecs)
 /* not really an SSDP "announce" as it is the response
  * to a SSDP "M-SEARCH" */
 static void
-SendSSDPAnnounce2(int s, struct sockaddr_in sockname, int st_no,
-                  const char * host, unsigned short port)
+SendSSDPResponse(int s, struct sockaddr_in sockname, int st_no,
+                  const char *host, unsigned short port)
 {
 	int l, n;
 	char buf[512];
+	char tmstr[30];
+	time_t tm = time(NULL);
+
 	/*
 	 * follow guideline from document "UPnP Device Architecture 1.0"
 	 * uppercase is recommended.
@@ -259,10 +218,7 @@ SendSSDPAnnounce2(int s, struct sockaddr_in sockname, int st_no,
 	 * SERVER: OS/ver UPnP/1.0 minidlna/1.0
 	 * - check what to put in the 'Cache-Control' header 
 	 * */
-	char   szTime[30];
-	time_t tTime = time(NULL);
-	strftime(szTime, 30,"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&tTime));
-
+	strftime(tmstr, sizeof(tmstr), "%a, %d %b %Y %H:%M:%S GMT", gmtime(&tm));
 	l = snprintf(buf, sizeof(buf), "HTTP/1.1 200 OK\r\n"
 		"CACHE-CONTROL: max-age=%u\r\n"
 		"DATE: %s\r\n"
@@ -274,38 +230,42 @@ SendSSDPAnnounce2(int s, struct sockaddr_in sockname, int st_no,
 		"Content-Length: 0\r\n"
 		"\r\n",
 		(runtime_vars.notify_interval<<1)+10,
-		szTime,
-		known_service_types[st_no], (st_no>1?"1":""),
-		uuidvalue, (st_no>0?"::":""), (st_no>0?known_service_types[st_no]:""), (st_no>1?"1":""),
+		tmstr,
+		known_service_types[st_no],
+		(st_no>1?"1":""),
+		uuidvalue,
+		(st_no > 0 ? "::" : ""),
+		(st_no > 0 ? known_service_types[st_no] : ""),
+		(st_no > 1 ? "1" : ""),
 		host, (unsigned int)port);
 	//DEBUG DPRINTF(E_DEBUG, L_SSDP, "Sending M-SEARCH response:\n%s", buf);
 	n = sendto(s, buf, l, 0,
 	           (struct sockaddr *)&sockname, sizeof(struct sockaddr_in) );
-	if(n < 0)
-	{
+	if (n < 0)
 		DPRINTF(E_ERROR, L_SSDP, "sendto(udp): %s\n", strerror(errno));
-	}
 }
 
-static void
-SendSSDPNotifies(int s, const char * host, unsigned short port,
-                 unsigned int lifetime)
+void
+SendSSDPNotifies(int s, const char *host, unsigned short port,
+                 unsigned int interval)
 {
 	struct sockaddr_in sockname;
 	int l, n, dup, i=0;
+	unsigned int lifetime;
 	char bufr[512];
 
 	memset(&sockname, 0, sizeof(struct sockaddr_in));
 	sockname.sin_family = AF_INET;
 	sockname.sin_port = htons(SSDP_PORT);
 	sockname.sin_addr.s_addr = inet_addr(SSDP_MCAST_ADDR);
+	lifetime = (interval << 1) + 10;
 
-	for( dup=0; dup<2; dup++ )
+	for (dup = 0; dup < 2; dup++)
 	{
-		if( dup )
+		if (dup)
 			_usleep(200000);
-		i=0;
-		while(known_service_types[i])
+		i = 0;
+		while (known_service_types[i])
 		{
 			l = snprintf(bufr, sizeof(bufr), 
 					"NOTIFY * HTTP/1.1\r\n"
@@ -320,41 +280,27 @@ SendSSDPNotifies(int s, const char * host, unsigned short port,
 					SSDP_MCAST_ADDR, SSDP_PORT,
 					lifetime,
 					host, port,
-					known_service_types[i], (i>1?"1":""),
-					uuidvalue, (i>0?"::":""), (i>0?known_service_types[i]:""), (i>1?"1":"") );
-			if(l>=sizeof(bufr))
+					known_service_types[i],
+					(i > 1 ? "1" : ""),
+					uuidvalue,
+					(i > 0 ? "::" : ""),
+					(i > 0 ? known_service_types[i] : ""),
+					(i > 1 ? "1" : ""));
+			if (l >= sizeof(bufr))
 			{
 				DPRINTF(E_WARN, L_SSDP, "SendSSDPNotifies(): truncated output\n");
 				l = sizeof(bufr);
 			}
-			//DEBUG DPRINTF(E_DEBUG, L_SSDP, "Sending NOTIFY:\n%s", bufr);
+			DPRINTF(E_MAXDEBUG, L_SSDP, "Sending ssdp:alive\n");
 			n = sendto(s, bufr, l, 0,
-				(struct sockaddr *)&sockname, sizeof(struct sockaddr_in) );
-			if(n < 0)
-			{
+				(struct sockaddr *)&sockname, sizeof(struct sockaddr_in));
+			if (n < 0)
 				DPRINTF(E_ERROR, L_SSDP, "sendto(udp_notify=%d, %s): %s\n", s, host, strerror(errno));
-			}
 			i++;
 		}
 	}
 }
 
-void
-SendSSDPNotifies2(int * sockets,
-                  unsigned short port,
-                  unsigned int lifetime)
-/*SendSSDPNotifies2(int * sockets, struct lan_addr_s * lan_addr, int n_lan_addr,
-                  unsigned short port,
-                  unsigned int lifetime)*/
-{
-	int i;
-	DPRINTF(E_DEBUG, L_SSDP, "Sending SSDP notifies\n");
-	for(i=0; i<n_lan_addr; i++)
-	{
-		SendSSDPNotifies(sockets[i], lan_addr[i].str, port, lifetime);
-	}
-}
-
 void
 ParseUPnPClient(char *location)
 {
@@ -368,8 +314,7 @@ ParseUPnPClient(char *location)
 	int content_len = sizeof(buf);
 	struct NameValueParserData xml;
 	int client;
-	enum client_types type = 0;
-	uint32_t flags = 0;
+	int type = 0;
 	char *model, *serial, *name;
 
 	if (strncmp(location, "http://", 7) != 0)
@@ -394,7 +339,7 @@ ParseUPnPClient(char *location)
 	dest.sin_port = htons(port);
 
 	s = socket(PF_INET, SOCK_STREAM, 0);
-	if( s < 0 )
+	if (s < 0)
 		return;
 
 	tv.tv_sec = 0;
@@ -402,114 +347,120 @@ ParseUPnPClient(char *location)
 	setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
 	setsockopt(s, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv));
 
-	if( connect(s, (struct sockaddr*)&dest, sizeof(struct sockaddr_in)) < 0 )
+	if (connect(s, (struct sockaddr*)&dest, sizeof(struct sockaddr_in)) < 0)
 		goto close;
 
 	n = snprintf(buf, sizeof(buf), "GET /%s HTTP/1.0\r\n"
 	                               "HOST: %s:%ld\r\n\r\n",
 	                               path, addr, port);
-	if( write(s, buf, n) < 1 )
+	if (write(s, buf, n) < 1)
 		goto close;
 
-	while( (n = read(s, buf+nread, sizeof(buf)-nread-1)) > 0 )
+	while ((n = read(s, buf+nread, sizeof(buf)-nread-1)) > 0)
 	{
 		nread += n;
 		buf[nread] = '\0';
 		n = nread;
 		p = buf;
 
-		while( !off && n-- > 0 )
+		while (!off && (n-- > 0))
 		{
-			if(p[0]=='\r' && p[1]=='\n' && p[2]=='\r' && p[3]=='\n')
+			if (p[0] == '\r' && p[1] == '\n' && p[2] == '\r' && p[3] == '\n')
 			{
 				off = p + 4;
 				do_headers = 1;
 			}
 			p++;
 		}
-		if( !off )
+		if (!off)
 			continue;
 
-		if( do_headers )
+		if (do_headers)
 		{
 			p = buf;
-			if( strncmp(p, "HTTP/", 5) != 0 )
+			if (strncmp(p, "HTTP/", 5) != 0)
 				goto close;
-			while(*p != ' ' && *p != '\t') p++;
+			while (*p != ' ' && *p != '\t')
+				p++;
 			/* If we don't get a 200 status, ignore it */
-			if( strtol(p, NULL, 10) != 200 )
+			if (strtol(p, NULL, 10) != 200)
 				goto close;
-			if( (p = strcasestr(p, "Content-Length:")) )
+			p = strcasestr(p, "Content-Length:");
+			if (p)
 				content_len = strtol(p+15, NULL, 10);
 			do_headers = 0;
 		}
-		if( buf + nread - off >= content_len )
+		if ((buf + nread - off) >= content_len)
 			break;
 	}
 close:
 	close(s);
-	if( !off )
+	if (!off)
 		return;
 	nread -= off - buf;
-	ParseNameValue(off, nread, &xml);
+	ParseNameValue(off, nread, &xml, 0);
 	model = GetValueFromNameValueList(&xml, "modelName");
 	serial = GetValueFromNameValueList(&xml, "serialNumber");
 	name = GetValueFromNameValueList(&xml, "friendlyName");
-	if( model )
+	if (model)
 	{
+		int i;
 		DPRINTF(E_DEBUG, L_SSDP, "Model: %s\n", model);
-		if( strstr(model, "Roku SoundBridge") )
+		for (i = 0; client_types[i].name; i++)
 		{
-			type = ERokuSoundBridge;
-			flags |= FLAG_MS_PFS;
-			flags |= FLAG_AUDIO_ONLY;
-			flags |= FLAG_MIME_WAV_WAV;
-		}
-		else if( strcmp(model, "Samsung DTV DMR") == 0 && serial )
-		{
-			DPRINTF(E_DEBUG, L_SSDP, "Serial: %s\n", serial);
-			/* The Series B I saw was 20081224DMR.  Series A should be older than that. */
-			if( atoi(serial) > 20081200 )
+			if (client_types[i].match_type != EModelName)
+				continue;
+			if (strstr(model, client_types[i].match) != NULL)
 			{
-				type = ESamsungSeriesB;
-				flags |= FLAG_SAMSUNG;
-				flags |= FLAG_DLNA;
-				flags |= FLAG_NO_RESIZE;
+				type = i;
+				break;
 			}
 		}
-		else
+
+		/* Special Samsung handling.  It's very hard to tell Series A from B */
+		if (type > 0 && client_types[type].type == ESamsungSeriesB)
 		{
-			if( name && (strcmp(name, "marantz DMP") == 0) )
+			if (serial)
 			{
-				type = EMarantzDMP;
-				flags |= FLAG_DLNA;
-				flags |= FLAG_MIME_WAV_WAV;
+				DPRINTF(E_DEBUG, L_SSDP, "Serial: %s\n", serial);
+				/* The Series B I saw was 20081224DMR.  Series A should be older than that. */
+				if (atoi(serial) < 20081201)
+					type = 0;
+			}
+			else
+			{
+				type = 0;
+			}
+		}
+
+		if (type == 0 && name != NULL)
+		{
+			for (i = 0; client_types[i].name; i++)
+			{
+				if (client_types[i].match_type != EFriendlyNameSSDP)
+					continue;
+				if (strcmp(name, client_types[i].match) == 0)
+				{
+					type = i;
+					break;
+				}
 			}
 		}
 	}
 	ClearNameValueList(&xml);
-	if( !type )
+	if (!type)
 		return;
 	/* Add this client to the cache if it's not there already. */
 	client = SearchClientCache(dest.sin_addr, 1);
-	if( client < 0 )
+	if (client < 0)
 	{
-		for( client=0; client<CLIENT_CACHE_SLOTS; client++ )
-		{
-			if( clients[client].addr.s_addr )
-				continue;
-			get_remote_mac(dest.sin_addr, clients[client].mac);
-			clients[client].addr = dest.sin_addr;
-			DPRINTF(E_DEBUG, L_SSDP, "Added client [%d/%s/%02X:%02X:%02X:%02X:%02X:%02X] to cache slot %d.\n",
-			                         type, inet_ntoa(clients[client].addr),
-			                         clients[client].mac[0], clients[client].mac[1], clients[client].mac[2],
-			                         clients[client].mac[3], clients[client].mac[4], clients[client].mac[5], client);
-			break;
-		}
+		AddClientCache(dest.sin_addr, type);
+	}
+	else
+	{
+		clients[client].type = type;
+		clients[client].age = time(NULL);
 	}
-	clients[client].type = type;
-	clients[client].flags = flags;
-	clients[client].age = time(NULL);
 }
 
 /* ProcessSSDPRequest()
@@ -528,68 +479,72 @@ ProcessSSDPRequest(int s, unsigned short port)
 
 	n = recvfrom(s, bufr, sizeof(bufr)-1, 0,
 	             (struct sockaddr *)&sendername, &len_r);
-	if(n < 0)
+	if (n < 0)
 	{
 		DPRINTF(E_ERROR, L_SSDP, "recvfrom(udp): %s\n", strerror(errno));
 		return;
 	}
 	bufr[n] = '\0';
+	n -= 2;
 
-	if(memcmp(bufr, "NOTIFY", 6) == 0)
+	if (memcmp(bufr, "NOTIFY", 6) == 0)
 	{
 		char *loc = NULL, *srv = NULL, *nts = NULL, *nt = NULL;
 		int loc_len = 0;
 		//DEBUG DPRINTF(E_DEBUG, L_SSDP, "Received SSDP notify:\n%.*s", n, bufr);
-		for(i=0; i < n; i++)
+		for (i = 0; i < n; i++)
 		{
 			if( bufr[i] == '*' )
 				break;
 		}
-		if( !strcasestrc(bufr+i, "HTTP/1.1", '\r') )
+		if (strcasestrc(bufr+i, "HTTP/1.1", '\r') == NULL)
 			return;
-		while(i < n)
+		while (i < n)
 		{
-			while((i < n - 2) && (bufr[i] != '\r' || bufr[i+1] != '\n'))
+			while ((i < n) && (bufr[i] != '\r' || bufr[i+1] != '\n'))
 				i++;
 			i += 2;
-			if(strncasecmp(bufr+i, "SERVER:", 7) == 0)
+			if (strncasecmp(bufr+i, "SERVER:", 7) == 0)
 			{
 				srv = bufr+i+7;
-				while(*srv == ' ' || *srv == '\t') srv++;
+				while (*srv == ' ' || *srv == '\t')
+					srv++;
 			}
-			else if(strncasecmp(bufr+i, "LOCATION:", 9) == 0)
+			else if (strncasecmp(bufr+i, "LOCATION:", 9) == 0)
 			{
 				loc = bufr+i+9;
-				while(*loc == ' ' || *loc == '\t') loc++;
-				while(loc[loc_len]!='\r' && loc[loc_len]!='\n') loc_len++;
+				while (*loc == ' ' || *loc == '\t')
+					loc++;
+				while (loc[loc_len]!='\r' && loc[loc_len]!='\n')
+					loc_len++;
 			}
-			else if(strncasecmp(bufr+i, "NTS:", 4) == 0)
+			else if (strncasecmp(bufr+i, "NTS:", 4) == 0)
 			{
 				nts = bufr+i+4;
-				while(*nts == ' ' || *nts == '\t') nts++;
+				while (*nts == ' ' || *nts == '\t')
+					nts++;
 			}
-			else if(strncasecmp(bufr+i, "NT:", 3) == 0)
+			else if (strncasecmp(bufr+i, "NT:", 3) == 0)
 			{
 				nt = bufr+i+3;
-				while(*nt == ' ' || *nt == '\t') nt++;
+				while(*nt == ' ' || *nt == '\t')
+					nt++;
 			}
 		}
-		if( !loc || !srv || !nt || !nts || (strncmp(nts, "ssdp:alive", 10) != 0) ||
-		    (strncmp(nt, "urn:schemas-upnp-org:device:MediaRenderer", 41) != 0) )
-		{
+		if (!loc || !srv || !nt || !nts || (strncmp(nts, "ssdp:alive", 10) != 0) ||
+		    (strncmp(nt, "urn:schemas-upnp-org:device:MediaRenderer", 41) != 0))
 			return;
-		}
 		loc[loc_len] = '\0';
-		if( (strncmp(srv, "Allegro-Software-RomPlug", 24) == 0) || /* Roku */
+		if ((strncmp(srv, "Allegro-Software-RomPlug", 24) == 0) || /* Roku */
 		    (strstr(loc, "SamsungMRDesc.xml") != NULL) || /* Samsung TV */
-		    (strstrc(srv, "DigiOn DiXiM", '\r') != NULL) ) /* Marantz Receiver */
+		    (strstrc(srv, "DigiOn DiXiM", '\r') != NULL)) /* Marantz Receiver */
 		{
 			/* Check if the client is already in cache */
 			i = SearchClientCache(sendername.sin_addr, 1);
-			if( i >= 0 )
+			if (i >= 0)
 			{
-				if( clients[i].type < EStandardDLNA150 &&
-				    clients[i].type != ESamsungSeriesA )
+				if (clients[i].type < EStandardDLNA150 &&
+				    clients[i].type != ESamsungSeriesA)
 				{
 					clients[i].age = time(NULL);
 					return;
@@ -598,100 +553,107 @@ ProcessSSDPRequest(int s, unsigned short port)
 			ParseUPnPClient(loc);
 		}
 	}
-	else if(memcmp(bufr, "M-SEARCH", 8) == 0)
+	else if (memcmp(bufr, "M-SEARCH", 8) == 0)
 	{
 		int st_len = 0, mx_len = 0, mx_val = 0;
 		//DPRINTF(E_DEBUG, L_SSDP, "Received SSDP request:\n%.*s\n", n, bufr);
-		for(i=0; i < n; i++)
+		for (i = 0; i < n; i++)
 		{
-			if( bufr[i] == '*' )
+			if (bufr[i] == '*')
 				break;
 		}
-		if( !strcasestrc(bufr+i, "HTTP/1.1", '\r') )
+		if (strcasestrc(bufr+i, "HTTP/1.1", '\r') == NULL)
 			return;
-		while(i < n)
+		while (i < n)
 		{
-			while((i < n - 2) && (bufr[i] != '\r' || bufr[i+1] != '\n'))
+			while ((i < n) && (bufr[i] != '\r' || bufr[i+1] != '\n'))
 				i++;
 			i += 2;
-			if(strncasecmp(bufr+i, "ST:", 3) == 0)
+			if (strncasecmp(bufr+i, "ST:", 3) == 0)
 			{
 				st = bufr+i+3;
 				st_len = 0;
-				while(*st == ' ' || *st == '\t') st++;
-				while(st[st_len]!='\r' && st[st_len]!='\n') st_len++;
+				while (*st == ' ' || *st == '\t')
+					st++;
+				while (st[st_len]!='\r' && st[st_len]!='\n')
+					st_len++;
 			}
-			else if(strncasecmp(bufr+i, "MX:", 3) == 0)
+			else if (strncasecmp(bufr+i, "MX:", 3) == 0)
 			{
 				mx = bufr+i+3;
 				mx_len = 0;
-				while(*mx == ' ' || *mx == '\t') mx++;
-				while(mx[mx_len]!='\r' && mx[mx_len]!='\n') mx_len++;
+				while (*mx == ' ' || *mx == '\t')
+					mx++;
+				while (mx[mx_len]!='\r' && mx[mx_len]!='\n')
+					mx_len++;
         			mx_val = strtol(mx, &mx_end, 10);
 			}
-			else if(strncasecmp(bufr+i, "MAN:", 4) == 0)
+			else if (strncasecmp(bufr+i, "MAN:", 4) == 0)
 			{
 				man = bufr+i+4;
 				man_len = 0;
-				while(*man == ' ' || *man == '\t') man++;
-				while(man[man_len]!='\r' && man[man_len]!='\n') man_len++;
+				while (*man == ' ' || *man == '\t')
+					man++;
+				while (man[man_len]!='\r' && man[man_len]!='\n')
+					man_len++;
 			}
 		}
 		/*DPRINTF(E_INFO, L_SSDP, "SSDP M-SEARCH packet received from %s:%d\n",
 	           inet_ntoa(sendername.sin_addr),
 	           ntohs(sendername.sin_port) );*/
-		if( GETFLAG(DLNA_STRICT_MASK) && (ntohs(sendername.sin_port) <= 1024 || ntohs(sendername.sin_port) == 1900) )
+		if (GETFLAG(DLNA_STRICT_MASK) && (ntohs(sendername.sin_port) <= 1024 || ntohs(sendername.sin_port) == 1900))
 		{
 			DPRINTF(E_INFO, L_SSDP, "WARNING: Ignoring invalid SSDP M-SEARCH from %s [bad source port %d]\n",
-			   inet_ntoa(sendername.sin_addr), ntohs(sendername.sin_port));
+				inet_ntoa(sendername.sin_addr), ntohs(sendername.sin_port));
 		}
-		else if( !man || (strncmp(man, "\"ssdp:discover\"", 15) != 0) )
+		else if (!man || (strncmp(man, "\"ssdp:discover\"", 15) != 0))
 		{
 			DPRINTF(E_INFO, L_SSDP, "WARNING: Ignoring invalid SSDP M-SEARCH from %s [bad %s header '%.*s']\n",
-			   inet_ntoa(sendername.sin_addr), "MAN", man_len, man);
+				inet_ntoa(sendername.sin_addr), "MAN", man_len, man);
 		}
-		else if( !mx || mx == mx_end || mx_val < 0 ) {
+		else if (!mx || mx == mx_end || mx_val < 0)
+		{
 			DPRINTF(E_INFO, L_SSDP, "WARNING: Ignoring invalid SSDP M-SEARCH from %s [bad %s header '%.*s']\n",
-			   inet_ntoa(sendername.sin_addr), "MX", mx_len, mx);
+				inet_ntoa(sendername.sin_addr), "MX", mx_len, mx);
 		}
-		else if( st && (st_len > 0) )
+		else if (st && (st_len > 0))
 		{
 			int l;
 			int lan_addr_index = 0;
 			/* find in which sub network the client is */
-			for(i = 0; i<n_lan_addr; i++)
+			for (i = 0; i < n_lan_addr; i++)
 			{
-				if( (sendername.sin_addr.s_addr & lan_addr[i].mask.s_addr)
-				   == (lan_addr[i].addr.s_addr & lan_addr[i].mask.s_addr))
+				if((sendername.sin_addr.s_addr & lan_addr[i].mask.s_addr) ==
+				   (lan_addr[i].addr.s_addr & lan_addr[i].mask.s_addr))
 				{
 					lan_addr_index = i;
 					break;
 				}
 			}
-			if( i == n_lan_addr )
+			if (n_lan_addr == i)
 			{
 				DPRINTF(E_DEBUG, L_SSDP, "Ignoring SSDP M-SEARCH on other interface [%s]\n",
 					inet_ntoa(sendername.sin_addr));
 				return;
 			}
 			DPRINTF(E_INFO, L_SSDP, "SSDP M-SEARCH from %s:%d ST: %.*s, MX: %.*s, MAN: %.*s\n",
-	        	   inet_ntoa(sendername.sin_addr),
-	           	   ntohs(sendername.sin_port),
-			   st_len, st, mx_len, mx, man_len, man);
+				inet_ntoa(sendername.sin_addr),
+				ntohs(sendername.sin_port),
+				st_len, st, mx_len, mx, man_len, man);
 			/* Responds to request with a device as ST header */
-			for(i = 0; known_service_types[i]; i++)
+			for (i = 0; known_service_types[i]; i++)
 			{
 				l = strlen(known_service_types[i]);
-				if( l <= st_len && (memcmp(st, known_service_types[i], l) == 0))
+				if ((l <= st_len) && (memcmp(st, known_service_types[i], l) == 0))
 				{
-					if( st_len != l )
+					if (st_len != l)
 					{
 						/* Check version number - must always be 1 currently. */
-						if( (st[l-1] == ':') && (st[l] == '1') )
+						if ((st[l-1] == ':') && (st[l] == '1'))
 							l++;
-						while( l < st_len )
+						while (l < st_len)
 						{
-							if( !isspace(st[l]) )
+							if (!isspace(st[l]))
 							{
 								DPRINTF(E_DEBUG, L_SSDP, "Ignoring SSDP M-SEARCH with bad extra data [%s]\n",
 									inet_ntoa(sendername.sin_addr));
@@ -699,25 +661,23 @@ ProcessSSDPRequest(int s, unsigned short port)
 							}
 							l++;
 						}
-						if( l != st_len )
+						if (l != st_len)
 							break;
 					}
 					_usleep(random()>>20);
-					SendSSDPAnnounce2(s, sendername,
-					                  i,
+					SendSSDPResponse(s, sendername, i,
 					                  lan_addr[lan_addr_index].str, port);
 					break;
 				}
 			}
 			/* Responds to request with ST: ssdp:all */
 			/* strlen("ssdp:all") == 8 */
-			if(st_len==8 && (0 == memcmp(st, "ssdp:all", 8)))
+			if ((st_len == 8) && (memcmp(st, "ssdp:all", 8) == 0))
 			{
-				for(i=0; known_service_types[i]; i++)
+				for (i=0; known_service_types[i]; i++)
 				{
-					l = (int)strlen(known_service_types[i]);
-					SendSSDPAnnounce2(s, sendername,
-					                  i,
+					l = strlen(known_service_types[i]);
+					SendSSDPResponse(s, sendername, i,
 					                  lan_addr[lan_addr_index].str, port);
 				}
 			}
@@ -725,9 +685,13 @@ ProcessSSDPRequest(int s, unsigned short port)
 		else
 		{
 			DPRINTF(E_INFO, L_SSDP, "Invalid SSDP M-SEARCH from %s:%d\n",
-	        	   inet_ntoa(sendername.sin_addr), ntohs(sendername.sin_port));
+				inet_ntoa(sendername.sin_addr), ntohs(sendername.sin_port));
 		}
 	}
+	else if (memcmp(bufr, "YOUKU-NOTIFY", 12) == 0)
+	{
+		return;
+	}
 	else
 	{
 		DPRINTF(E_WARN, L_SSDP, "Unknown udp packet received from %s:%d\n",
@@ -738,7 +702,7 @@ ProcessSSDPRequest(int s, unsigned short port)
 /* This will broadcast ssdp:byebye notifications to inform 
  * the network that UPnP is going down. */
 int
-SendSSDPGoodbye(int * sockets, int n_sockets)
+SendSSDPGoodbyes(void)
 {
 	struct sockaddr_in sockname;
 	int n, l;
@@ -753,9 +717,9 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
 
 	for (dup = 0; dup < 2; dup++)
 	{
-		for(j=0; j<n_sockets; j++)
+		for (j = 0; j < n_lan_addr; j++)
 		{
-			for(i=0; known_service_types[i]; i++)
+			for (i = 0; known_service_types[i]; i++)
 			{
 				l = snprintf(bufr, sizeof(bufr),
 				             "NOTIFY * HTTP/1.1\r\n"
@@ -767,12 +731,12 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
 				             SSDP_MCAST_ADDR, SSDP_PORT,
 				             known_service_types[i], (i>1?"1":""),
 				             uuidvalue, (i>0?"::":""), (i>0?known_service_types[i]:""), (i>1?"1":"") );
-				//DEBUG DPRINTF(E_DEBUG, L_SSDP, "Sending NOTIFY:\n%s", bufr);
-				n = sendto(sockets[j], bufr, l, 0,
+				DPRINTF(E_MAXDEBUG, L_SSDP, "Sending ssdp:byebye\n");
+				n = sendto(lan_addr[j].snotify, bufr, l, 0,
 				           (struct sockaddr *)&sockname, sizeof(struct sockaddr_in) );
-				if(n < 0)
+				if (n < 0)
 				{
-					DPRINTF(E_ERROR, L_SSDP, "sendto(udp_shutdown=%d): %s\n", sockets[j], strerror(errno));
+					DPRINTF(E_ERROR, L_SSDP, "sendto(udp_shutdown=%d): %s\n", lan_addr[j].snotify, strerror(errno));
 					ret = -1;
 					break;
 				}
@@ -786,35 +750,39 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
  * register services offered by MiniUPnPd to a running instance of
  * MiniSSDPd */
 int
-SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
+SubmitServicesToMiniSSDPD(const char *host, unsigned short port)
+{
 	struct sockaddr_un addr;
 	int s;
 	unsigned char buffer[2048];
 	char strbuf[256];
-	unsigned char * p;
+	unsigned char *p;
 	int i, l;
 
 	s = socket(AF_UNIX, SOCK_STREAM, 0);
-	if(s < 0) {
+	if (s < 0)
+	{
 		DPRINTF(E_ERROR, L_SSDP, "socket(unix): %s", strerror(errno));
 		return -1;
 	}
 	addr.sun_family = AF_UNIX;
 	strncpy(addr.sun_path, minissdpdsocketpath, sizeof(addr.sun_path));
-	if(connect(s, (struct sockaddr *)&addr, sizeof(struct sockaddr_un)) < 0) {
+	if (connect(s, (struct sockaddr *)&addr, sizeof(struct sockaddr_un)) < 0)
+	{
 		DPRINTF(E_ERROR, L_SSDP, "connect(\"%s\"): %s",
 		        minissdpdsocketpath, strerror(errno));
 		return -1;
 	}
-	for(i = 0; known_service_types[i]; i++) {
+	for (i = 0; known_service_types[i]; i++)
+	{
 		buffer[0] = 4;
 		p = buffer + 1;
-		l = (int)strlen(known_service_types[i]);
-		if(i > 0)
+		l = strlen(known_service_types[i]);
+		if (i > 0)
 			l++;
 		CODELENGTH(l, p);
 		memcpy(p, known_service_types[i], l);
-		if(i > 0)
+		if (i > 0)
 			p[l-1] = '1';
 		p += l;
 		l = snprintf(strbuf, sizeof(strbuf), "%s::%s%s", 
@@ -822,7 +790,7 @@ SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
 		CODELENGTH(l, p);
 		memcpy(p, strbuf, l);
 		p += l;
-		l = (int)strlen(MINIDLNA_SERVER_STRING);
+		l = strlen(MINIDLNA_SERVER_STRING);
 		CODELENGTH(l, p);
 		memcpy(p, MINIDLNA_SERVER_STRING, l);
 		p += l;
@@ -831,7 +799,8 @@ SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
 		CODELENGTH(l, p);
 		memcpy(p, strbuf, l);
 		p += l;
-		if(write(s, buffer, p - buffer) < 0) {
+		if(write(s, buffer, p - buffer) < 0)
+		{
 			DPRINTF(E_ERROR, L_SSDP, "write(): %s", strerror(errno));
 			return -1;
 		}
