@@ -40,7 +40,6 @@ enum upnpconfigoptions {
 	UPNPPORT,			/* port */
 	UPNPPRESENTATIONURL,		/* presentation_url */
 	UPNPNOTIFY_INTERVAL,		/* notify_interval */
-	UPNPSYSTEM_UPTIME,		/* system_uptime */
 	UPNPUUID,			/* uuid */
 	UPNPSERIAL,			/* serial */
 	UPNPMODEL_NAME,			/* model_name */
@@ -55,7 +54,8 @@ enum upnpconfigoptions {
 	UPNPMINISSDPDSOCKET,		/* minissdpdsocket */
 	ENABLE_TIVO,			/* enable support for streaming images and music to TiVo */
 	ENABLE_DLNA_STRICT,		/* strictly adhere to DLNA specs */
-	ROOT_CONTAINER			/* root ObjectID (instead of "0") */
+	ROOT_CONTAINER,			/* root ObjectID (instead of "0") */
+	USER_ACCOUNT			/* user account to run as */
 };
 
 /* readoptionsfile()
