@@ -23,6 +23,7 @@
 #include <locale.h>
 #include <libgen.h>
 #include <inttypes.h>
+#include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -33,7 +34,9 @@
 #include <libintl.h>
 #endif
 #include <sqlite3.h>
+#include "libav.h"
 
+#include "scanner_sqlite.h"
 #include "upnpglobalvars.h"
 #include "metadata.h"
 #include "playlist.h"
@@ -43,20 +46,29 @@
 #include "albumart.h"
 #include "log.h"
 
+#if SCANDIR_CONST
+typedef const struct dirent scan_filter;
+#else
+typedef struct dirent scan_filter;
+#endif
+#ifndef AV_LOG_PANIC
+#define AV_LOG_PANIC AV_LOG_FATAL
+#endif
+
 int valid_cache = 0;
 
 struct virtual_item
 {
-	sqlite3_int64 objectID;
+	int64_t objectID;
 	char parentID[64];
 	char name[256];
 };
 
-sqlite_int64
-get_next_available_id(const char * table, const char * parentID)
+int64_t
+get_next_available_id(const char *table, const char *parentID)
 {
 		char *ret, *base;
-		sqlite_int64 objectID = 0;
+		int64_t objectID = 0;
 
 		ret = sql_get_text_field(db, "SELECT OBJECT_ID from %s where ID = "
 		                             "(SELECT max(ID) from %s where PARENT_ID = '%s')",
@@ -73,13 +85,12 @@ get_next_available_id(const char * table, const char * parentID)
 }
 
 int
-insert_container(const char * item, const char * rootParent, const char * refID, const char *class,
-                 const char *artist, const char *genre, const char *album_art, sqlite3_int64 *objectID, sqlite3_int64 *parentID)
+insert_container(const char *item, const char *rootParent, const char *refID, const char *class,
+                 const char *artist, const char *genre, const char *album_art, int64_t *objectID, int64_t *parentID)
 {
 	char *result;
 	char *base;
 	int ret = 0;
-	sqlite_int64 detailID = 0;
 
 	result = sql_get_text_field(db, "SELECT OBJECT_ID from OBJECTS o "
 	                                "left join DETAILS d on (o.DETAIL_ID = d.ID)"
@@ -99,6 +110,7 @@ insert_container(const char * item, const char * rootParent, const char * refID,
 	}
 	else
 	{
+		int64_t detailID = 0;
 		*objectID = 0;
 		*parentID = get_next_available_id("OBJECTS", rootParent);
 		if( refID )
@@ -114,8 +126,9 @@ insert_container(const char * item, const char * rootParent, const char * refID,
 		ret = sql_exec(db, "INSERT into OBJECTS"
 		                   " (OBJECT_ID, PARENT_ID, REF_ID, DETAIL_ID, CLASS, NAME) "
 		                   "VALUES"
-		                   " ('%s$%"PRIX64"', '%s', %Q, %"PRId64", 'container.%s', '%q')",
-		                   rootParent, *parentID, rootParent, refID, detailID, class, item);
+		                   " ('%s$%llX', '%s', %Q, %lld, 'container.%s', '%q')",
+		                   rootParent, (long long)*parentID, rootParent,
+		                   refID, (long long)detailID, class, item);
 	}
 	sqlite3_free(result);
 
@@ -123,40 +136,36 @@ insert_container(const char * item, const char * rootParent, const char * refID,
 }
 
 static void
-insert_containers(const char * name, const char *path, const char * refID, const char * class, sqlite3_int64 detailID)
+insert_containers(const char *name, const char *path, const char *refID, const char *class, int64_t detailID)
 {
 	char sql[128];
 	char **result;
 	int ret;
 	int cols, row;
-	sqlite_int64 objectID, parentID;
+	int64_t objectID, parentID;
 
 	if( strstr(class, "imageItem") )
 	{
-		char *date = NULL, *cam = NULL;
-		char date_taken[13], camera[64];
+		char *date_taken = NULL, *camera = NULL;
 		static struct virtual_item last_date;
 		static struct virtual_item last_cam;
 		static struct virtual_item last_camdate;
-		static sqlite_int64 last_all_objectID = 0;
+		static long long last_all_objectID = 0;
 
-		snprintf(sql, sizeof(sql), "SELECT DATE, CREATOR from DETAILS where ID = %lld", detailID);
+		snprintf(sql, sizeof(sql), "SELECT DATE, CREATOR from DETAILS where ID = %lld", (long long)detailID);
 		ret = sql_get_table(db, sql, &result, &row, &cols);
 		if( ret == SQLITE_OK )
 		{
-			date = result[2];
-			cam = result[3];
+			date_taken = result[2];
+			camera = result[3];
 		}
-
-		if( date )
-		{
-			strncpy(date_taken, date, 10);
+		if( date_taken )
 			date_taken[10] = '\0';
-		}
 		else
-		{
-			strcpy(date_taken, _("Unknown Date"));
-		}
+			date_taken = _("Unknown Date");
+		if( !camera )
+			camera = _("Unknown Camera");
+
 		if( valid_cache && strcmp(last_date.name, date_taken) == 0 )
 		{
 			last_date.objectID++;
@@ -165,31 +174,23 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		else
 		{
 			insert_container(date_taken, IMAGE_DATE_ID, NULL, "album.photoAlbum", NULL, NULL, NULL, &objectID, &parentID);
-			sprintf(last_date.parentID, IMAGE_DATE_ID"$%"PRIX64, parentID);
+			sprintf(last_date.parentID, IMAGE_DATE_ID"$%llX", (unsigned long long)parentID);
 			last_date.objectID = objectID;
-			strcpy(last_date.name, date_taken);
+			strncpyt(last_date.name, date_taken, sizeof(last_date.name));
 			//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached date item: %s/%s/%X\n", last_date.name, last_date.parentID, last_date.objectID);
 		}
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
-		             last_date.parentID, last_date.objectID, last_date.parentID, refID, class, detailID, name);
+		             " ('%s$%llX', '%s', '%s', '%s', %lld, %Q)",
+		             last_date.parentID, (long long)last_date.objectID, last_date.parentID, refID, class, (long long)detailID, name);
 
-		if( cam )
-		{
-			strncpy(camera, cam, 63);
-			camera[63] = '\0';
-		}
-		else
-		{
-			strcpy(camera, _("Unknown Camera"));
-		}
 		if( !valid_cache || strcmp(camera, last_cam.name) != 0 )
 		{
 			insert_container(camera, IMAGE_CAMERA_ID, NULL, "storageFolder", NULL, NULL, NULL, &objectID, &parentID);
-			sprintf(last_cam.parentID, IMAGE_CAMERA_ID"$%"PRIX64, parentID);
-			strncpy(last_cam.name, camera, 255);
+			sprintf(last_cam.parentID, IMAGE_CAMERA_ID"$%llX", (long long)parentID);
+			strncpyt(last_cam.name, camera, sizeof(last_cam.name));
+			/* Invalidate last_camdate cache */
 			last_camdate.name[0] = '\0';
 		}
 		if( valid_cache && strcmp(last_camdate.name, date_taken) == 0 )
@@ -200,16 +201,16 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		else
 		{
 			insert_container(date_taken, last_cam.parentID, NULL, "album.photoAlbum", NULL, NULL, NULL, &objectID, &parentID);
-			sprintf(last_camdate.parentID, "%s$%"PRIX64, last_cam.parentID, parentID);
+			sprintf(last_camdate.parentID, "%s$%llX", last_cam.parentID, (long long)parentID);
 			last_camdate.objectID = objectID;
-			strcpy(last_camdate.name, date_taken);
+			strncpyt(last_camdate.name, date_taken, sizeof(last_camdate.name));
 			//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached camdate item: %s/%s/%s/%X\n", camera, last_camdate.name, last_camdate.parentID, last_camdate.objectID);
 		}
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
-		             last_camdate.parentID, last_camdate.objectID, last_camdate.parentID, refID, class, detailID, name);
+		             " ('%s$%llX', '%s', '%s', '%s', %lld, %Q)",
+		             last_camdate.parentID, last_camdate.objectID, last_camdate.parentID, refID, class, (long long)detailID, name);
 		/* All Images */
 		if( !last_all_objectID )
 		{
@@ -218,12 +219,12 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('"IMAGE_ALL_ID"$%"PRIX64"', '"IMAGE_ALL_ID"', '%s', '%s', %"PRId64", %Q)",
-		             last_all_objectID++, refID, class, detailID, name);
+		             " ('"IMAGE_ALL_ID"$%llX', '"IMAGE_ALL_ID"', '%s', '%s', %lld, %Q)",
+		             last_all_objectID++, refID, class, (long long)detailID, name);
 	}
 	else if( strstr(class, "audioItem") )
 	{
-		snprintf(sql, sizeof(sql), "SELECT ALBUM, ARTIST, GENRE, ALBUM_ART from DETAILS where ID = %lld", detailID);
+		snprintf(sql, sizeof(sql), "SELECT ALBUM, ARTIST, GENRE, ALBUM_ART from DETAILS where ID = %lld", (long long)detailID);
 		ret = sql_get_table(db, sql, &result, &row, &cols);
 		if( ret != SQLITE_OK )
 			return;
@@ -241,7 +242,7 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		static struct virtual_item last_genre;
 		static struct virtual_item last_genreArtist;
 		static struct virtual_item last_genreArtistAll;
-		static sqlite_int64 last_all_objectID = 0;
+		static long long last_all_objectID = 0;
 
 		if( album )
 		{
@@ -252,29 +253,29 @@ insert_containers(const char * name, const char *path, const char * refID, const
 			}
 			else
 			{
-				strcpy(last_album.name, album);
+				strncpyt(last_album.name, album, sizeof(last_album.name));
 				insert_container(album, MUSIC_ALBUM_ID, NULL, "album.musicAlbum", artist, genre, album_art, &objectID, &parentID);
-				sprintf(last_album.parentID, MUSIC_ALBUM_ID"$%llX", parentID);
+				sprintf(last_album.parentID, MUSIC_ALBUM_ID"$%llX", (long long)parentID);
 				last_album.objectID = objectID;
 				//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached album item: %s/%s/%X\n", last_album.name, last_album.parentID, last_album.objectID);
 			}
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
-			             last_album.parentID, last_album.objectID, last_album.parentID, refID, class, detailID, name);
+			             " ('%s$%llX', '%s', '%s', '%s', %lld, %Q)",
+			             last_album.parentID, last_album.objectID, last_album.parentID, refID, class, (long long)detailID, name);
 		}
 		if( artist )
 		{
 			if( !valid_cache || strcmp(artist, last_artist.name) != 0 )
 			{
 				insert_container(artist, MUSIC_ARTIST_ID, NULL, "person.musicArtist", NULL, genre, NULL, &objectID, &parentID);
-				sprintf(last_artist.parentID, MUSIC_ARTIST_ID"$%"PRIX64, parentID);
-				strcpy(last_artist.name, artist);
+				sprintf(last_artist.parentID, MUSIC_ARTIST_ID"$%llX", (long long)parentID);
+				strncpyt(last_artist.name, artist, sizeof(last_artist.name));
 				last_artistAlbum.name[0] = '\0';
 				/* Add this file to the "- All Albums -" container as well */
 				insert_container(_("- All Albums -"), last_artist.parentID, NULL, "album", artist, genre, NULL, &objectID, &parentID);
-				sprintf(last_artistAlbumAll.parentID, "%s$%"PRIX64, last_artist.parentID, parentID);
+				sprintf(last_artistAlbumAll.parentID, "%s$%llX", last_artist.parentID, (long long)parentID);
 				last_artistAlbumAll.objectID = objectID;
 			}
 			else
@@ -290,33 +291,32 @@ insert_containers(const char * name, const char *path, const char * refID, const
 			{
 				insert_container(album?album:_("Unknown Album"), last_artist.parentID, album?last_album.parentID:NULL,
 				                 "album.musicAlbum", artist, genre, album_art, &objectID, &parentID);
-				sprintf(last_artistAlbum.parentID, "%s$%"PRIX64, last_artist.parentID, parentID);
+				sprintf(last_artistAlbum.parentID, "%s$%llX", last_artist.parentID, (long long)parentID);
 				last_artistAlbum.objectID = objectID;
-				strcpy(last_artistAlbum.name, album?album:_("Unknown Album"));
+				strncpyt(last_artistAlbum.name, album ? album : _("Unknown Album"), sizeof(last_artistAlbum.name));
 				//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached artist/album item: %s/%s/%X\n", last_artist.name, last_artist.parentID, last_artist.objectID);
 			}
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
-			             last_artistAlbum.parentID, last_artistAlbum.objectID, last_artistAlbum.parentID, refID, class, detailID, name);
+			             " ('%s$%llX', '%s', '%s', '%s', %lld, %Q)",
+			             last_artistAlbum.parentID, last_artistAlbum.objectID, last_artistAlbum.parentID, refID, class, (long long)detailID, name);
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
-			             last_artistAlbumAll.parentID, last_artistAlbumAll.objectID, last_artistAlbumAll.parentID, refID, class, detailID, name);
+			             " ('%s$%llX', '%s', '%s', '%s', %lld, %Q)",
+			             last_artistAlbumAll.parentID, last_artistAlbumAll.objectID, last_artistAlbumAll.parentID, refID, class, (long long)detailID, name);
 		}
 		if( genre )
 		{
 			if( !valid_cache || strcmp(genre, last_genre.name) != 0 )
 			{
 				insert_container(genre, MUSIC_GENRE_ID, NULL, "genre.musicGenre", NULL, NULL, NULL, &objectID, &parentID);
-				sprintf(last_genre.parentID, MUSIC_GENRE_ID"$%"PRIX64, parentID);
-				strcpy(last_genre.name, genre);
-				last_genreArtist.name[0] = '\0';
+				sprintf(last_genre.parentID, MUSIC_GENRE_ID"$%llX", (long long)parentID);
+				strncpyt(last_genre.name, genre, sizeof(last_genre.name));
 				/* Add this file to the "- All Artists -" container as well */
 				insert_container(_("- All Artists -"), last_genre.parentID, NULL, "person", NULL, genre, NULL, &objectID, &parentID);
-				sprintf(last_genreArtistAll.parentID, "%s$%"PRIX64, last_genre.parentID, parentID);
+				sprintf(last_genreArtistAll.parentID, "%s$%llX", last_genre.parentID, (long long)parentID);
 				last_genreArtistAll.objectID = objectID;
 			}
 			else
@@ -331,21 +331,21 @@ insert_containers(const char * name, const char *path, const char * refID, const
 			{
 				insert_container(artist?artist:_("Unknown Artist"), last_genre.parentID, artist?last_artist.parentID:NULL,
 				                 "person.musicArtist", NULL, genre, NULL, &objectID, &parentID);
-				sprintf(last_genreArtist.parentID, "%s$%"PRIX64, last_genre.parentID, parentID);
+				sprintf(last_genreArtist.parentID, "%s$%llX", last_genre.parentID, (long long)parentID);
 				last_genreArtist.objectID = objectID;
-				strcpy(last_genreArtist.name, artist?artist:_("Unknown Artist"));
+				strncpyt(last_genreArtist.name, artist ? artist : _("Unknown Artist"), sizeof(last_genreArtist.name));
 				//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached genre/artist item: %s/%s/%X\n", last_genreArtist.name, last_genreArtist.parentID, last_genreArtist.objectID);
 			}
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
-			             last_genreArtist.parentID, last_genreArtist.objectID, last_genreArtist.parentID, refID, class, detailID, name);
+			             " ('%s$%llX', '%s', '%s', '%s', %lld, %Q)",
+			             last_genreArtist.parentID, last_genreArtist.objectID, last_genreArtist.parentID, refID, class, (long long)detailID, name);
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
-			             last_genreArtistAll.parentID, last_genreArtistAll.objectID, last_genreArtistAll.parentID, refID, class, detailID, name);
+			             " ('%s$%llX', '%s', '%s', '%s', %lld, %Q)",
+			             last_genreArtistAll.parentID, last_genreArtistAll.objectID, last_genreArtistAll.parentID, refID, class, (long long)detailID, name);
 		}
 		/* All Music */
 		if( !last_all_objectID )
@@ -355,12 +355,12 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('"MUSIC_ALL_ID"$%"PRIX64"', '"MUSIC_ALL_ID"', '%s', '%s', %"PRId64", %Q)",
-		             last_all_objectID++, refID, class, detailID, name);
+		             " ('"MUSIC_ALL_ID"$%llX', '"MUSIC_ALL_ID"', '%s', '%s', %lld, %Q)",
+		             last_all_objectID++, refID, class, (long long)detailID, name);
 	}
 	else if( strstr(class, "videoItem") )
 	{
-		static sqlite_int64 last_all_objectID = 0;
+		static long long last_all_objectID = 0;
 
 		/* All Videos */
 		if( !last_all_objectID )
@@ -370,8 +370,8 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('"VIDEO_ALL_ID"$%"PRIX64"', '"VIDEO_ALL_ID"', '%s', '%s', %"PRId64", %Q)",
-		             last_all_objectID++, refID, class, detailID, name);
+		             " ('"VIDEO_ALL_ID"$%llX', '"VIDEO_ALL_ID"', '%s', '%s', %lld, %Q)",
+		             last_all_objectID++, refID, class, (long long)detailID, name);
 		return;
 	}
 	else
@@ -383,27 +383,22 @@ insert_containers(const char * name, const char *path, const char * refID, const
 }
 
 int
-insert_directory(const char * name, const char * path, const char * base, const char * parentID, int objectID)
+insert_directory(const char *name, const char *path, const char *base, const char *parentID, int objectID)
 {
-	int found = 0;
-	sqlite_int64 detailID = 0;
-	char * refID = NULL;
+	int64_t detailID = 0;
 	char class[] = "container.storageFolder";
 	char *result, *p;
 	static char last_found[256] = "-1";
 
 	if( strcmp(base, BROWSEDIR_ID) != 0 )
 	{
-		if( asprintf(&refID, "%s%s$%X", BROWSEDIR_ID, parentID, objectID) == -1 )
-			return 1;
-	}
-
-	if( refID )
-	{
-		char id_buf[64], parent_buf[64];
+		int found = 0;
+		char id_buf[64], parent_buf[64], refID[64];
 		char *dir_buf, *dir;
+
  		dir_buf = strdup(path);
 		dir = dirname(dir_buf);
+		snprintf(refID, sizeof(refID), "%s%s$%X", BROWSEDIR_ID, parentID, objectID);
 		snprintf(id_buf, sizeof(id_buf), "%s%s$%X", base, parentID, objectID);
 		snprintf(parent_buf, sizeof(parent_buf), "%s%s", base, parentID);
 		while( !found )
@@ -425,7 +420,7 @@ insert_directory(const char * name, const char * path, const char * base, const
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, DETAIL_ID, CLASS, NAME) "
 			             "VALUES"
-			             " ('%s', '%s', %Q, %"PRId64", '%s', '%q')",
+			             " ('%s', '%s', %Q, %lld, '%s', '%q')",
 			             id_buf, parent_buf, refID, detailID, class, strrchr(dir, '/')+1);
 			if( (p = strrchr(id_buf, '$')) )
 				*p = '\0';
@@ -435,34 +430,29 @@ insert_directory(const char * name, const char * path, const char * base, const
 				*p = '\0';
 			dir = dirname(dir);
 		}
-		free(refID);
 		free(dir_buf);
 		return 0;
 	}
 
 	detailID = GetFolderMetadata(name, path, NULL, NULL, find_album_art(path, NULL, 0));
 	sql_exec(db, "INSERT into OBJECTS"
-	             " (OBJECT_ID, PARENT_ID, REF_ID, DETAIL_ID, CLASS, NAME) "
+	             " (OBJECT_ID, PARENT_ID, DETAIL_ID, CLASS, NAME) "
 	             "VALUES"
-	             " ('%s%s$%X', '%s%s', %Q, %"PRId64", '%s', '%q')",
-	             base, parentID, objectID, base, parentID, refID, detailID, class, name);
-	if( refID )
-		free(refID);
-
+	             " ('%s%s$%X', '%s%s', %lld, '%s', '%q')",
+	             base, parentID, objectID, base, parentID, detailID, class, name);
 	return 0;
 }
 
 int
-insert_file(char * name, const char * path, const char * parentID, int object)
+insert_file(char *name, const char *path, const char *parentID, int object)
 {
 	char class[32];
 	char objectID[64];
-	sqlite3_int64 detailID = 0;
+	int64_t detailID = 0;
 	char base[8];
-	char * typedir_parentID;
-	int typedir_objectID;
-	char * baseid;
-	char * orig_name = NULL;
+	char *typedir_parentID;
+	char *baseid;
+	char *orig_name = NULL;
 
 	if( is_image(name) )
 	{
@@ -492,8 +482,7 @@ insert_file(char * name, const char * path, const char * parentID, int object)
 		strcpy(class, "item.audioItem.musicTrack");
 		detailID = GetAudioMetadata(path, name);
 	}
-	if( orig_name )
-		free(orig_name);
+	free(orig_name);
 	if( !detailID )
 	{
 		DPRINTF(E_WARN, L_SCANNER, "Unsuccessful getting details for %s!\n", path);
@@ -505,12 +494,12 @@ insert_file(char * name, const char * path, const char * parentID, int object)
 	sql_exec(db, "INSERT into OBJECTS"
 	             " (OBJECT_ID, PARENT_ID, CLASS, DETAIL_ID, NAME) "
 	             "VALUES"
-	             " ('%s', '%s%s', '%s', %"PRId64", '%q')",
+	             " ('%s', '%s%s', '%s', %lld, '%q')",
 	             objectID, BROWSEDIR_ID, parentID, class, detailID, name);
 
 	if( *parentID )
 	{
-		typedir_objectID = 0;
+		int typedir_objectID = 0;
 		typedir_parentID = strdup(parentID);
 		baseid = strrchr(typedir_parentID, '$');
 		if( baseid )
@@ -524,7 +513,7 @@ insert_file(char * name, const char * path, const char * parentID, int object)
 	sql_exec(db, "INSERT into OBJECTS"
 	             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 	             "VALUES"
-	             " ('%s%s$%X', '%s%s', '%s', '%s', %"PRId64", '%q')",
+	             " ('%s%s$%X', '%s%s', '%s', '%s', %lld, '%q')",
 	             base, parentID, object, base, parentID, objectID, class, detailID, name);
 
 	insert_containers(name, path, objectID, class, detailID);
@@ -535,100 +524,50 @@ int
 CreateDatabase(void)
 {
 	int ret, i;
-	const char * containers[] = { "0","-1",   "root",
-	                         MUSIC_ID, "0", _("Music"),
-	                     MUSIC_ALL_ID, MUSIC_ID, _("All Music"),
-	                   MUSIC_GENRE_ID, MUSIC_ID, _("Genre"),
-	                  MUSIC_ARTIST_ID, MUSIC_ID, _("Artist"),
-	                   MUSIC_ALBUM_ID, MUSIC_ID, _("Album"),
-	                     MUSIC_DIR_ID, MUSIC_ID, _("Folders"),
-	                   MUSIC_PLIST_ID, MUSIC_ID, _("Playlists"),
+	const char *containers[] = { "0","-1",   "root",
+	                        MUSIC_ID, "0", _("Music"),
+	                    MUSIC_ALL_ID, MUSIC_ID, _("All Music"),
+	                  MUSIC_GENRE_ID, MUSIC_ID, _("Genre"),
+	                 MUSIC_ARTIST_ID, MUSIC_ID, _("Artist"),
+	                  MUSIC_ALBUM_ID, MUSIC_ID, _("Album"),
+	                    MUSIC_DIR_ID, MUSIC_ID, _("Folders"),
+	                  MUSIC_PLIST_ID, MUSIC_ID, _("Playlists"),
 
-	                         VIDEO_ID, "0", _("Video"),
-	                     VIDEO_ALL_ID, VIDEO_ID, _("All Video"),
-	                     VIDEO_DIR_ID, VIDEO_ID, _("Folders"),
+	                        VIDEO_ID, "0", _("Video"),
+	                    VIDEO_ALL_ID, VIDEO_ID, _("All Video"),
+	                    VIDEO_DIR_ID, VIDEO_ID, _("Folders"),
 
-	                         IMAGE_ID, "0", _("Pictures"),
-	                     IMAGE_ALL_ID, IMAGE_ID, _("All Pictures"),
-	                    IMAGE_DATE_ID, IMAGE_ID, _("Date Taken"),
-	                  IMAGE_CAMERA_ID, IMAGE_ID, _("Camera"),
-	                     IMAGE_DIR_ID, IMAGE_ID, _("Folders"),
+	                        IMAGE_ID, "0", _("Pictures"),
+	                    IMAGE_ALL_ID, IMAGE_ID, _("All Pictures"),
+	                   IMAGE_DATE_ID, IMAGE_ID, _("Date Taken"),
+	                 IMAGE_CAMERA_ID, IMAGE_ID, _("Camera"),
+	                    IMAGE_DIR_ID, IMAGE_ID, _("Folders"),
 
-	                     BROWSEDIR_ID, "0", _("Browse Folders"),
+	                    BROWSEDIR_ID, "0", _("Browse Folders"),
 			0 };
 
-	ret = sql_exec(db, "CREATE TABLE OBJECTS ( "
-					"ID INTEGER PRIMARY KEY AUTOINCREMENT, "
-					"OBJECT_ID TEXT UNIQUE NOT NULL, "
-					"PARENT_ID TEXT NOT NULL, "
-					"REF_ID TEXT DEFAULT NULL, "
-					"CLASS TEXT NOT NULL, "
-					"DETAIL_ID INTEGER DEFAULT NULL, "
-					"NAME TEXT DEFAULT NULL"
-					");");
+	ret = sql_exec(db, create_objectTable_sqlite);
 	if( ret != SQLITE_OK )
 		goto sql_failed;
-	ret = sql_exec(db, "CREATE TABLE DETAILS ( "
-					"ID INTEGER PRIMARY KEY AUTOINCREMENT, "
-					"PATH TEXT DEFAULT NULL, "
-					"SIZE INTEGER, "
-					"TIMESTAMP INTEGER, "
-					"TITLE TEXT COLLATE NOCASE, "
-					"DURATION TEXT, "
-					"BITRATE INTEGER, "
-					"SAMPLERATE INTEGER, "
-					"CREATOR TEXT COLLATE NOCASE, "
-					"ARTIST TEXT COLLATE NOCASE, "
-					"ALBUM TEXT COLLATE NOCASE, "
-					"GENRE TEXT COLLATE NOCASE, "
-					"COMMENT TEXT, "
-					"CHANNELS INTEGER, "
-					"DISC INTEGER, "
-					"TRACK INTEGER, "
-					"DATE DATE, "
-					"RESOLUTION TEXT, "
-					"THUMBNAIL BOOL DEFAULT 0, "
-					"ALBUM_ART INTEGER DEFAULT 0, "
-					"ROTATION INTEGER, "
-					"DLNA_PN TEXT, "
-					"MIME TEXT"
-					")");
+	ret = sql_exec(db, create_detailTable_sqlite);
 	if( ret != SQLITE_OK )
 		goto sql_failed;
-	ret = sql_exec(db, "CREATE TABLE ALBUM_ART ( "
-					"ID INTEGER PRIMARY KEY AUTOINCREMENT, "
-					"PATH TEXT NOT NULL"
-					")");
+	ret = sql_exec(db, create_albumArtTable_sqlite);
 	if( ret != SQLITE_OK )
 		goto sql_failed;
-	ret = sql_exec(db, "CREATE TABLE CAPTIONS ("
-					"ID INTEGER PRIMARY KEY, "
-					"PATH TEXT NOT NULL"
-					")");
+	ret = sql_exec(db, create_captionTable_sqlite);
 	if( ret != SQLITE_OK )
 		goto sql_failed;
-	ret = sql_exec(db, "CREATE TABLE BOOKMARKS ("
-					"ID INTEGER PRIMARY KEY, "
-					"SEC INTEGER"
-					")");
+	ret = sql_exec(db, create_bookmarkTable_sqlite);
 	if( ret != SQLITE_OK )
 		goto sql_failed;
-	ret = sql_exec(db, "CREATE TABLE PLAYLISTS ("
-					"ID INTEGER PRIMARY KEY AUTOINCREMENT, "
-					"NAME TEXT NOT NULL, "
-					"PATH TEXT NOT NULL, "
-					"ITEMS INTEGER DEFAULT 0, "
-					"FOUND INTEGER DEFAULT 0"
-					")");
+	ret = sql_exec(db, create_playlistTable_sqlite);
 	if( ret != SQLITE_OK )
 		goto sql_failed;
-	ret = sql_exec(db, "CREATE TABLE SETTINGS ("
-					"UPDATE_ID INTEGER PRIMARY KEY DEFAULT 0, "
-					"FLAGS INTEGER DEFAULT 0"
-					")");
+	ret = sql_exec(db, create_settingsTable_sqlite);
 	if( ret != SQLITE_OK )
 		goto sql_failed;
-	ret = sql_exec(db, "INSERT into SETTINGS values (0, 0)");
+	ret = sql_exec(db, "INSERT into SETTINGS values ('UPDATE_ID', '0')");
 	if( ret != SQLITE_OK )
 		goto sql_failed;
 	for( i=0; containers[i]; i=i+3 )
@@ -655,96 +594,141 @@ sql_failed:
 	return (ret != SQLITE_OK);
 }
 
-int
-filter_audio(const struct dirent *d)
+static inline int
+filter_hidden(scan_filter *d)
 {
-	return ( (*d->d_name != '.') &&
-	         ((d->d_type == DT_DIR) ||
-	          (d->d_type == DT_LNK) ||
-	          (d->d_type == DT_UNKNOWN) ||
+	return (d->d_name[0] != '.');
+}
+
+static int
+filter_type(scan_filter *d)
+{
+	return ( (d->d_type == DT_DIR) ||
+	         (d->d_type == DT_LNK) ||
+	         (d->d_type == DT_UNKNOWN)
+	       );
+}
+
+static int
+filter_a(scan_filter *d)
+{
+	return ( filter_hidden(d) &&
+	         (filter_type(d) ||
 		  ((d->d_type == DT_REG) &&
 		   (is_audio(d->d_name) ||
-	            is_playlist(d->d_name)
-		   )
-	       ) ));
+	            is_playlist(d->d_name))))
+	       );
 }
 
-int
-filter_video(const struct dirent *d)
+static int
+filter_av(scan_filter *d)
 {
-	return ( (*d->d_name != '.') &&
-	         ((d->d_type == DT_DIR) ||
-	          (d->d_type == DT_LNK) ||
-	          (d->d_type == DT_UNKNOWN) ||
+	return ( filter_hidden(d) &&
+	         (filter_type(d) ||
 		  ((d->d_type == DT_REG) &&
-		   is_video(d->d_name) )
-	       ) );
+		   (is_audio(d->d_name) ||
+		    is_video(d->d_name) ||
+	            is_playlist(d->d_name))))
+	       );
 }
 
-int
-filter_images(const struct dirent *d)
+static int
+filter_ap(scan_filter *d)
 {
-	return ( (*d->d_name != '.') &&
-	         ((d->d_type == DT_DIR) ||
-	          (d->d_type == DT_LNK) ||
-	          (d->d_type == DT_UNKNOWN) ||
+	return ( filter_hidden(d) &&
+	         (filter_type(d) ||
 		  ((d->d_type == DT_REG) &&
-		   is_image(d->d_name) )
-	       ) );
+		   (is_audio(d->d_name) ||
+		    is_image(d->d_name) ||
+	            is_playlist(d->d_name))))
+	       );
 }
 
-int
-filter_media(const struct dirent *d)
+static int
+filter_v(scan_filter *d)
 {
-	return ( (*d->d_name != '.') &&
-	         ((d->d_type == DT_DIR) ||
-	          (d->d_type == DT_LNK) ||
-	          (d->d_type == DT_UNKNOWN) ||
-	          ((d->d_type == DT_REG) &&
-	           (is_image(d->d_name) ||
-	            is_audio(d->d_name) ||
-	            is_video(d->d_name) ||
-	            is_playlist(d->d_name)
-	           )
-	       ) ));
+	return ( filter_hidden(d) &&
+	         (filter_type(d) ||
+		  (d->d_type == DT_REG &&
+	           is_video(d->d_name)))
+	       );
+}
+
+static int
+filter_vp(scan_filter *d)
+{
+	return ( filter_hidden(d) &&
+	         (filter_type(d) ||
+		  ((d->d_type == DT_REG) &&
+		   (is_video(d->d_name) ||
+	            is_image(d->d_name))))
+	       );
+}
+
+static int
+filter_p(scan_filter *d)
+{
+	return ( filter_hidden(d) &&
+	         (filter_type(d) ||
+		  (d->d_type == DT_REG &&
+		   is_image(d->d_name)))
+	       );
+}
+
+static int
+filter_avp(scan_filter *d)
+{
+	return ( filter_hidden(d) &&
+	         (filter_type(d) ||
+		  ((d->d_type == DT_REG) &&
+		   (is_audio(d->d_name) ||
+		    is_image(d->d_name) ||
+		    is_video(d->d_name) ||
+	            is_playlist(d->d_name))))
+	       );
 }
 
 void
-ScanDirectory(const char * dir, const char * parent, enum media_types dir_type)
+ScanDirectory(const char *dir, const char *parent, media_types dir_types)
 {
 	struct dirent **namelist;
 	int i, n, startID=0;
 	char parent_id[PATH_MAX];
 	char full_path[PATH_MAX];
-	char * name = NULL;
+	char *name = NULL;
 	static long long unsigned int fileno = 0;
 	enum file_types type;
 
-	setlocale(LC_COLLATE, "");
-	if( chdir(dir) != 0 )
-		return;
-
 	DPRINTF(parent?E_INFO:E_WARN, L_SCANNER, _("Scanning %s\n"), dir);
-	switch( dir_type )
+	switch( dir_types )
 	{
 		case ALL_MEDIA:
-			n = scandir(".", &namelist, filter_media, alphasort);
+			n = scandir(dir, &namelist, filter_avp, alphasort);
 			break;
-		case AUDIO_ONLY:
-			n = scandir(".", &namelist, filter_audio, alphasort);
+		case TYPE_AUDIO:
+			n = scandir(dir, &namelist, filter_a, alphasort);
 			break;
-		case VIDEO_ONLY:
-			n = scandir(".", &namelist, filter_video, alphasort);
+		case TYPE_AUDIO|TYPE_VIDEO:
+			n = scandir(dir, &namelist, filter_av, alphasort);
 			break;
-		case IMAGES_ONLY:
-			n = scandir(".", &namelist, filter_images, alphasort);
+		case TYPE_AUDIO|TYPE_IMAGES:
+			n = scandir(dir, &namelist, filter_ap, alphasort);
+			break;
+		case TYPE_VIDEO:
+			n = scandir(dir, &namelist, filter_v, alphasort);
+			break;
+		case TYPE_VIDEO|TYPE_IMAGES:
+			n = scandir(dir, &namelist, filter_vp, alphasort);
+			break;
+		case TYPE_IMAGES:
+			n = scandir(dir, &namelist, filter_p, alphasort);
 			break;
 		default:
 			n = -1;
 			break;
 	}
 	if (n < 0) {
-		fprintf(stderr, "Error scanning %s [scandir]\n", dir);
+		DPRINTF(E_WARN, L_SCANNER, "Error scanning %s\n", dir);
 		return;
 	}
 
@@ -772,13 +756,13 @@ ScanDirectory(const char * dir, const char * parent, enum media_types dir_type)
 		}
 		else
 		{
-			type = resolve_unknown_type(full_path, dir_type);
+			type = resolve_unknown_type(full_path, dir_types);
 		}
 		if( (type == TYPE_DIR) && (access(full_path, R_OK|X_OK) == 0) )
 		{
 			insert_directory(name, full_path, BROWSEDIR_ID, (parent ? parent:""), i+startID);
 			sprintf(parent_id, "%s$%X", (parent ? parent:""), i+startID);
-			ScanDirectory(full_path, parent_id, dir_type);
+			ScanDirectory(full_path, parent_id, dir_types);
 		}
 		else if( type == TYPE_FILE && (access(full_path, R_OK) == 0) )
 		{
@@ -789,11 +773,7 @@ ScanDirectory(const char * dir, const char * parent, enum media_types dir_type)
 		free(namelist[i]);
 	}
 	free(namelist);
-	if( parent )
-	{
-		chdir(dirname((char*)dir));
-	}
-	else
+	if( !parent )
 	{
 		DPRINTF(E_WARN, L_SCANNER, _("Scanning %s finished (%llu files)!\n"), dir, fileno);
 	}
@@ -802,26 +782,32 @@ ScanDirectory(const char * dir, const char * parent, enum media_types dir_type)
 void
 start_scanner()
 {
-	struct media_dir_s * media_path = media_dirs;
-	char name[PATH_MAX];
+	struct media_dir_s *media_path = media_dirs;
+	char name[MAXPATHLEN];
 
 	if (setpriority(PRIO_PROCESS, 0, 15) == -1)
 		DPRINTF(E_WARN, L_INOTIFY,  "Failed to reduce scanner thread priority\n");
 
 #ifdef READYNAS
-	FILE * flag = fopen("/ramfs/.upnp-av_scan", "w");
+	FILE *flag = fopen("/ramfs/.upnp-av_scan", "w");
 	if( flag )
 		fclose(flag);
 #endif
-	freopen("/dev/null", "a", stderr);
+	setlocale(LC_COLLATE, "");
+
+	av_register_all();
+	av_log_set_level(AV_LOG_PANIC);
 	while( media_path )
 	{
-		strncpy(name, media_path->path, sizeof(name));
-		GetFolderMetadata(basename(name), media_path->path, NULL, NULL, 0);
-		ScanDirectory(media_path->path, NULL, media_path->type);
+		int64_t id;
+		strncpyt(name, media_path->path, sizeof(name));
+		id = GetFolderMetadata(basename(name), media_path->path, NULL, NULL, 0);
+		/* Use TIMESTAMP to store the media type */
+		sql_exec(db, "UPDATE DETAILS set TIMESTAMP = %d where ID = %lld", media_path->types, (long long)id);
+		ScanDirectory(media_path->path, NULL, media_path->types);
+		sql_exec(db, "INSERT into SETTINGS values (%Q, %Q)", "media_dir", media_path->path);
 		media_path = media_path->next;
 	}
-	freopen("/proc/self/fd/2", "a", stderr);
 #ifdef READYNAS
 	if( access("/ramfs/.rescan_done", F_OK) == 0 )
 		system("/bin/sh /ramfs/.rescan_done");
